char * const Class_aliases[] = {
  "mt",
  "mntner",
  "pn",
  "person",
  "ro",
  "role",
  "rt",
  "route",
  "as",
  "as-set",
  "rs",
  "route-set",
  "fs",
  "filter-set",
  "is",
  "rtr-set",
  "ps",
  "peering-set",
  "an",
  "aut-num",
  "ir",
  "inet-rtr",
  "ak",
  "as-block",
  "in",
  "inetnum",
  "kc",
  "key-cert",
  NULL
}; /* Class_aliases */
