/* generated by 'RPSLSyntax.java' - DO NOT HAND-EDIT */
#ifndef SYNTAX_TAB_H
#define SYNTAX_TAB_H

extern void members_is_reset();
extern int members_isparse();
extern void members_rs_reset();
extern int members_rsparse();
extern void filter_reset();
extern int filterparse();
extern void components_reset();
extern int componentsparse();
extern void aggr_mtd_reset();
extern int aggr_mtdparse();
extern void aggr_bndry_reset();
extern int aggr_bndryparse();
extern void default_reset();
extern int defaultparse();
extern void export_reset();
extern int exportparse();
extern void import_reset();
extern int importparse();
extern void ifaddr_reset();
extern int ifaddrparse();
extern void inject_reset();
extern int injectparse();
extern void peering_reset();
extern int peeringparse();
extern void peer_reset();
extern int peerparse();

syntax_t syntax_tab[] = {
    {
        /* name */
        "filter-set",
        /* core_regex_pattern */
        "^.{1,80}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^((AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])|fltr-[A-Z0-9_-]*[A-Z0-9]):)*fltr-[A-Z0-9_-]*[A-Z0-9](:(AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])|fltr-[A-Z0-9_-]*[A-Z0-9]))*$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "as-set",
        /* core_regex_pattern */
        "^.{1,80}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^((AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])|as-[A-Z0-9_-]*[A-Z0-9]):)*as-[A-Z0-9_-]*[A-Z0-9](:(AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])|as-[A-Z0-9_-]*[A-Z0-9]))*$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "route-set",
        /* core_regex_pattern */
        "^.{1,80}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^((AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])|rs-[A-Z0-9_-]*[A-Z0-9]):)*rs-[A-Z0-9_-]*[A-Z0-9](:(AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])|rs-[A-Z0-9_-]*[A-Z0-9]))*$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "peering-set",
        /* core_regex_pattern */
        "^.{1,80}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^((AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])|prng-[A-Z0-9_-]*[A-Z0-9]):)*prng-[A-Z0-9_-]*[A-Z0-9](:(AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])|prng-[A-Z0-9_-]*[A-Z0-9]))*$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "rtr-set",
        /* core_regex_pattern */
        "^.{1,80}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^((AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])|rtrs-[A-Z0-9_-]*[A-Z0-9]):)*rtrs-[A-Z0-9_-]*[A-Z0-9](:(AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])|rtrs-[A-Z0-9_-]*[A-Z0-9]))*$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "members-as",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^((((AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])|as-[A-Z0-9_-]*[A-Z0-9]):)*as-[A-Z0-9_-]*[A-Z0-9](:(AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])|as-[A-Z0-9_-]*[A-Z0-9]))*)|AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "members-is",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        members_is_reset,
        /* front_end_parser */
        members_isparse
    },
    {
        /* name */
        "members-rs",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        members_rs_reset,
        /* front_end_parser */
        members_rsparse
    },
    {
        /* name */
        "mbrs-by-ref",
        /* core_regex_pattern */
        "^.{1,80}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^[A-Z]([A-Z0-9_-]*[A-Z0-9])?$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        "^(as-any|rs-any|peeras|and|or|not|atomic|from|to|at|action|accept|announce|except|refine|networks|into|inbound|outbound|rs-.*|rtrs-.*|fltr-.*|prng-.*)$",
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "free-form",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "nic-handle",
        /* core_regex_pattern */
        "^.{1,30}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^(([A-Z]{2,4}[0-9]{0,6}(-[A-Z]{1,9})?)|(AUTO-[0-9]+([A-Z]{2,4})?))$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "object-name",
        /* core_regex_pattern */
        "^.{1,80}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^[A-Z]([A-Z0-9_-]*[A-Z0-9])?$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        "^(any|as-any|rs-any|peeras|and|or|not|atomic|from|to|at|action|accept|announce|except|refine|networks|into|inbound|outbound|as-.*|rs-.*|rtrs-.*|fltr-.*|prng-.*)$",
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "e-mail",
        /* core_regex_pattern */
        "^.{1,80}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^(([^][()<>,;:\\\\\"[:space:]]+)|(\"[^\"@\\\\]+\"))@([A-Z0-9-]+(\\.[A-Z0-9-]+)+)$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "changed",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^(([^][()<>,;:\\\\\"[:space:]]+)|(\"[^\"@\\\\]+\"))@([A-Z0-9-]+(\\.[A-Z0-9-]+)+)( [0-9]{8})?$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "filter",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        filter_reset,
        /* front_end_parser */
        filterparse
    },
    {
        /* name */
        "registry-name",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^[A-Z]([A-Z0-9_-]*[A-Z0-9])?$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "domain-name",
        /* core_regex_pattern */
        "^.{1,254}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^[A-Z0-9]([-A-Z0-9]*[A-Z0-9])?(\\.[A-Z0-9]([-A-Z0-9]*[A-Z0-9])?)*$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "person-name",
        /* core_regex_pattern */
        "^[A-Z0-9.\\\\`'_-]{1,64}( [A-Z0-9.\\\\`'_-]{1,64}){0,9}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^[A-Z]([A-Z0-9.`'_-]*[A-Z0-9`'_-])?([[:space:]]+[A-Z0-9.`'_-]+)*[[:space:]]+[A-Z]([A-Z0-9.`'_-]*[A-Z0-9`'_-])?$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        "(^(Dr|Prof|Mv|Ms|Mr)\\.?[[:space:]])|([[:space:]](Dr|Prof|Mv|Ms|Mr)\\.?[[:space:]])",
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "telephone-number",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^\\+[[:space:]]*[0-9][0-9.[:space:]-]*(\\([0-9.[:space:]-]*[0-9][0-9.[:space:]-]*\\))?([0-9.[:space:]-]*[0-9][0-9.[:space:]-]*)?([[:space:]]+ext.[0-9.[:space:]-]*[0-9][0-9.[:space:]-]*)?$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "auth-scheme",
        /* core_regex_pattern */
        "^.{0,90}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^(MAIL-FROM [^[:space:]]{1,80}|CRYPT-PW [A-Z0-9./]{13}|PGPKEY-[A-F0-9]{8})$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "inetnum",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^((([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])) - ((([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "mnt-routes",
        /* core_regex_pattern */
        "^[A-Z]([A-Z0-9_-]){1,80}([[:space:]]+.*)*$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^[A-Z]([A-Z0-9_-]*[A-Z0-9])?$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        "^(peeras|and|or|not|atomic|from|to|at|action|accept|announce|except|refine|networks|into|inbound|outbound|as-.*|rs-.*|rtrs-.*|fltr-.*|prng-.*)$",
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "public-key",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "fingerpr",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^(([A-F0-9]{4} ){9}[A-F0-9]{4})|(([A-F0-9]{2} ){15}[A-F0-9]{2})$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "key-cert",
        /* core_regex_pattern */
        "^PGPKEY-[A-F0-9]{8}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^PGPKEY-[A-F0-9]{8}$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "method",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^PGP$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "address-prefix",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^((([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))/([12]?[0-9]|3[012])$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "as-number",
        /* core_regex_pattern */
        "^.{1,7}$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-4])$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "as-block",
        /* core_regex_pattern */
        "^AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-4]) - AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-4])$",
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        "^AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-4]) - AS([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-4])$",
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        NULL,
        /* front_end_parser */
        NULL 
    },
    {
        /* name */
        "components",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        components_reset,
        /* front_end_parser */
        componentsparse
    },
    {
        /* name */
        "aggr-mtd",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        aggr_mtd_reset,
        /* front_end_parser */
        aggr_mtdparse
    },
    {
        /* name */
        "aggr-bndry",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        aggr_bndry_reset,
        /* front_end_parser */
        aggr_bndryparse
    },
    {
        /* name */
        "default",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        default_reset,
        /* front_end_parser */
        defaultparse
    },
    {
        /* name */
        "export",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        export_reset,
        /* front_end_parser */
        exportparse
    },
    {
        /* name */
        "import",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        import_reset,
        /* front_end_parser */
        importparse
    },
    {
        /* name */
        "ifaddr",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        ifaddr_reset,
        /* front_end_parser */
        ifaddrparse
    },
    {
        /* name */
        "inject",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        inject_reset,
        /* front_end_parser */
        injectparse
    },
    {
        /* name */
        "peering",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        peering_reset,
        /* front_end_parser */
        peeringparse
    },
    {
        /* name */
        "peer",
        /* core_regex_pattern */
        NULL,
        /* core_regex, set by syntax_init() */
        NULL,
        /* core_reserved_regex_pattern */
        NULL,
        /* core_reserved_regex, set by syntax_init() */
        NULL,
        /* core_parser_reset */
        NULL,
        /* core_parser */
        NULL,
        /* front_end_regex_pattern */
        NULL,
        /* front_end_regex, set by syntax_init() */
        NULL,
        /* front_end_reserved_regex_pattern */
        NULL,
        /* front_end_reserved_regex, set by syntax_init() */
        NULL,
        /* front_end_parser_reset */
        peer_reset,
        /* front_end_parser */
        peerparse
    }
};

#endif /* SYNTAX_TAB_H */
