int const Class_dbase_code_map[] = {
  0,
  1,
  2,
  9999,   /* XXX: no class code to map */
  4,
  9999,   /* XXX: no class code to map */
  6,
  7,
  9999,   /* XXX: no class code to map */
  9,
  10,
  11,
  12,
  13,
  14,
  15,
  16,
  NULL
}; /* Class_dbase_code_map */
