char * const Class_codes[] = {
  "ak",
  "as",
  "an",
  "",   /* XXX: no class code */
  "ir",
  "",   /* XXX: no class code */
  "in",
  "kc",
  "",   /* XXX: no class code */
  "mt",
  "pn",
  "ro",
  "rt",
  "rs",
  "fs",
  "ps",
  "is",
  NULL
}; /* Class_codes */
