/************************************
Copyright (c) 2000,2001,2002                    RIPE NCC

All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the author not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
***************************************/


#define CA_BINDPORT 		0
#define CA_HELP 		1
#define CA_REPLYBANNER 		2
#define CA_SOURCE 		3
#define CA_TESTMODE 		4
#define CA_OVERRIDECRYPTEDPW 		5
#define CA_WHOISHELP 		6
#define CA_AUTODBMHELP 		7
#define CA_ATTDESC 		8
#define CA_DEFLOOK 		9
#define CA_DBFILE 		10
#define CA_CANUPD 		11
#define CA_CROSSNOTIFYSOURCES 		12
#define CA_UID 		13
#define CA_QRYLOG 		14
#define CA_QRY2LOG 		15
#define CA_ERRLOG 		16
#define CA_AUDITLOG 		17
#define CA_UPDLOG 		18
#define CA_ACKLOG 		19
#define CA_LOCKDIR 		20
#define CA_PIDFILE 		21
#define CA_UPDATELOCK 		22
#define CA_CLEANLOCK 		23
#define CA_SERIALDIR 		24
#define CA_OLDSERIALDIR 		25
#define CA_SERIALINCOMINGDIR 		26
#define CA_KEEPOPEN 		27
#define CA_ALLOCMNT 		28
#define CA_GETUPDATESFROM 		29
#define CA_WHOISUPDFROM 		30
#define CA_DENYWHOISACCESS 		31
#define CA_TMPDIR 		32
#define CA_PGPV 		33
#define CA_PGPK 		34
#define CA_PGPPATH 		35
#define CA_MAILCMD 		36
#define CA_OVERFLOWSIZE 		37
#define CA_DBCACHESIZE 		38
#define CA_NROFNAMES 		39
#define CA_DEFMAIL 		40
#define CA_HUMAILBOX 		41
#define CA_AUTOBOX 		42
#define CA_DUPLICATEPERSONCHECK 		43
#define CA_NOMATCH 		44
#define CA_CN_SUBJECT_ADD 		45
#define CA_CN_SUBJECT_DEL 		46
#define CA_CN_EXPLAIN_ADD 		47
#define CA_CN_EXPLAIN_DEL 		48
#define CA_CN_OVERLAP_ADD 		49
#define CA_CN_OVERLAP_DEL 		50
#define CA_CNO_SUBJECT_ADD 		51
#define CA_CNO_SUBJECT_DEL 		52
#define CA_CNO_EXPLAIN_ADD 		53
#define CA_CNO_EXPLAIN_DEL 		54
#define CA_CNO_OVERLAP_ADD 		55
#define CA_CNO_OVERLAP_DEL 		56
#define CA_MAILTXT 		57
#define CA_MHEADER 		58
#define CA_FAILURETXT 		59
#define CA_SUCCESSTXT 		60
#define CA_HELPHEADER 		61
#define CA_ACKERR 		62
#define CA_ACKOK 		63
#define CA_ACKSIG 		64
#define CA_NOTITXT 		65
#define CA_NOTIMAILTXT 		66
#define CA_NOTINETWORKTXT 		67
#define CA_NHEADER 		68
#define CA_FWHEADER 		69
#define CA_FWTXT 		70
#define CA_FWMAILTXT 		71
#define CA_FWNETWORKTXT 		72
#define CA_MTFWHEADER 		73
#define CA_MTFWTXT 		74
#define CA_DENYACCESSTXT 		75
#define CA_REFERRALTXT 		76
#define CA_REFERRALERRORTXT 		77
#define CA_REFERRALLOOPERRORTXT 		78
#define CA_REFERRALTIMEOUTTXT 		79
#define CA_REFERRALTRUNCTXT 		80
#define CA_REFERRALENDTXT 		81
#define CA_REFERRALTIMEOUT 		82
#define CA_REFERRALMAXLINES 		83
#define CA_RIGHTS 		84
#define CA_COUNTRY 		85
#define CA_RIPADMIN	86
#define CA_NOTIFLOG	87
#define CA_AUTHMETHOD 		88
