typedef enum _C_Type_t {
  C_ANY = -1, 
  C_AK,
  C_AS,
  C_AN,
  C_BOGUS0,  /* XXX: no class code to enumerate */
  C_IR,
  C_BOGUS1,  /* XXX: no class code to enumerate */
  C_IN,
  C_KC,
  C_BOGUS2,  /* XXX: no class code to enumerate */
  C_MT,
  C_PN,
  C_RO,
  C_RT,
  C_RS,
  C_FS,
  C_PS,
  C_IS,
  C_END
} C_Type_t;

#define C_DN 127    /* XXX: class required by software */
#define NO_C_DN
#define C_I6 126    /* XXX: class required by software */
#define NO_C_I6
#define C_IT 125    /* XXX: class required by software */
#define NO_C_IT
