#define yy_create_buffer export_create_buffer
#define yy_delete_buffer export_delete_buffer
#define yy_scan_buffer export_scan_buffer
#define yy_scan_string export_scan_string
#define yy_scan_bytes export_scan_bytes
#define yy_flex_debug export_flex_debug
#define yy_init_buffer export_init_buffer
#define yy_flush_buffer export_flush_buffer
#define yy_load_buffer_state export_load_buffer_state
#define yy_switch_to_buffer export_switch_to_buffer
#define yyin exportin
#define yyleng exportleng
#define yylex exportlex
#define yyout exportout
#define yyrestart exportrestart
#define yytext exporttext
#define yywrap exportwrap

#line 20 "export.lex.c"
/* A lexical scanner generated by flex */

/* Scanner skeleton version:
 * $Header: /ncc/cvsroot/ncc/RIP/modules/rpsl/export.lex.c,v 1.2 2002/02/20 16:41:48 shane Exp $
 */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5

#include <stdio.h>


/* cfront 1.2 defines "c_plusplus" instead of "__cplusplus" */
#ifdef c_plusplus
#ifndef __cplusplus
#define __cplusplus
#endif
#endif


#ifdef __cplusplus

#include <stdlib.h>
#include <unistd.h>

/* Use prototypes in function declarations. */
#define YY_USE_PROTOS

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

#if __STDC__

#define YY_USE_PROTOS
#define YY_USE_CONST

#endif	/* __STDC__ */
#endif	/* ! __cplusplus */

#ifdef __TURBOC__
 #pragma warn -rch
 #pragma warn -use
#include <io.h>
#include <stdlib.h>
#define YY_USE_CONST
#define YY_USE_PROTOS
#endif

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif


#ifdef YY_USE_PROTOS
#define YY_PROTO(proto) proto
#else
#define YY_PROTO(proto) ()
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yy_start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart( yyin )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#define YY_BUF_SIZE 16384

typedef struct yy_buffer_state *YY_BUFFER_STATE;

extern int yyleng;
extern FILE *yyin, *yyout;

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

/* The funky do-while in the following #define is used to turn the definition
 * int a single C statement (which needs a semi-colon terminator).  This
 * avoids problems with code like:
 *
 * 	if ( condition_holds )
 *		yyless( 5 );
 *	else
 *		do_something_else();
 *
 * Prior to using the do-while the compiler would get upset at the
 * "else" because it interpreted the "if" statement as being all
 * done when it reached the ';' after the yyless() call.
 */

/* Return all but the first 'n' matched characters back to the input stream. */

#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		*yy_cp = yy_hold_char; \
		YY_RESTORE_YY_MORE_OFFSET \
		yy_c_buf_p = yy_cp = yy_bp + n - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, yytext_ptr )

/* The following is because we cannot portably get our hands on size_t
 * (without autoconf's help, which isn't available because we want
 * flex-generated scanners to compile on their own).
 */
typedef unsigned int yy_size_t;


struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;
#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2
	};

static YY_BUFFER_STATE yy_current_buffer = 0;

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 */
#define YY_CURRENT_BUFFER yy_current_buffer


/* yy_hold_char holds the character lost when yytext is formed. */
static char yy_hold_char;

static int yy_n_chars;		/* number of characters read into yy_ch_buf */


int yyleng;

/* Points to current character in buffer. */
static char *yy_c_buf_p = (char *) 0;
static int yy_init = 1;		/* whether we need to initialize */
static int yy_start = 0;	/* start state number */

/* Flag which is used to allow yywrap()'s to do buffer switches
 * instead of setting up a fresh yyin.  A bit of a hack ...
 */
static int yy_did_buffer_switch_on_eof;

void yyrestart YY_PROTO(( FILE *input_file ));

void yy_switch_to_buffer YY_PROTO(( YY_BUFFER_STATE new_buffer ));
void yy_load_buffer_state YY_PROTO(( void ));
YY_BUFFER_STATE yy_create_buffer YY_PROTO(( FILE *file, int size ));
void yy_delete_buffer YY_PROTO(( YY_BUFFER_STATE b ));
void yy_init_buffer YY_PROTO(( YY_BUFFER_STATE b, FILE *file ));
void yy_flush_buffer YY_PROTO(( YY_BUFFER_STATE b ));
#define YY_FLUSH_BUFFER yy_flush_buffer( yy_current_buffer )

YY_BUFFER_STATE yy_scan_buffer YY_PROTO(( char *base, yy_size_t size ));
YY_BUFFER_STATE yy_scan_string YY_PROTO(( yyconst char *yy_str ));
YY_BUFFER_STATE yy_scan_bytes YY_PROTO(( yyconst char *bytes, int len ));

static void *yy_flex_alloc YY_PROTO(( yy_size_t ));
static void *yy_flex_realloc YY_PROTO(( void *, yy_size_t ));
static void yy_flex_free YY_PROTO(( void * ));

#define yy_new_buffer yy_create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! yy_current_buffer ) \
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
	yy_current_buffer->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! yy_current_buffer ) \
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE ); \
	yy_current_buffer->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (yy_current_buffer->yy_at_bol)

typedef unsigned char YY_CHAR;
FILE *yyin = (FILE *) 0, *yyout = (FILE *) 0;
typedef int yy_state_type;
extern char *yytext;
#define yytext_ptr yytext

static yy_state_type yy_get_previous_state YY_PROTO(( void ));
static yy_state_type yy_try_NUL_trans YY_PROTO(( yy_state_type current_state ));
static int yy_get_next_buffer YY_PROTO(( void ));
static void yy_fatal_error YY_PROTO(( yyconst char msg[] ));

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yytext_ptr = yy_bp; \
	yyleng = (int) (yy_cp - yy_bp); \
	yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
	yy_c_buf_p = yy_cp;

#define YY_NUM_RULES 55
#define YY_END_OF_BUFFER 56
static yyconst short int yy_accept[321] =
    {   0,
        0,    0,   56,   54,    1,    1,   54,   52,    6,   53,
       54,   53,   53,   53,   53,   53,   53,   53,   53,   53,
       53,   53,   53,   53,   54,   54,    1,    7,    0,    0,
       52,    0,   53,    5,    9,    8,   10,   53,   53,   53,
       53,   25,   53,   53,   53,   53,   53,   53,   53,   53,
       53,   53,   53,   53,   53,    2,   53,   53,   53,   53,
       53,   53,   53,   53,   53,   14,    0,   38,   39,    0,
        0,   51,    0,   53,    3,   53,   12,   53,    0,   43,
       53,   53,   30,   53,   53,   53,   53,   33,   53,   53,
       53,   53,    0,   32,   53,   53,    0,    4,   53,   53,

       53,   53,   53,   53,   53,   30,    0,   53,   53,    0,
        0,   11,   53,   53,   53,    0,   45,    0,    0,   53,
       53,   53,   37,   53,   53,    0,   53,   27,   53,   53,
       53,    0,    0,   53,    0,   31,   53,   53,   53,   53,
       53,    0,   46,    0,   17,   53,    0,    0,   53,   53,
       53,    0,   45,    0,    0,    0,    0,   53,   53,   53,
       53,    0,    0,   53,    0,    0,    0,   53,   53,   53,
       53,    0,   53,   53,    0,   46,   53,    0,   40,    0,
       15,   53,   18,    0,    0,    0,    0,    0,    0,    0,
       34,   53,   53,   19,    0,   44,    0,    0,   53,    0,

        0,    0,   13,   53,    0,   47,    0,   53,   28,    0,
        0,    0,    0,   42,   50,   53,    0,    0,    0,    0,
        0,    0,    0,    0,   53,   53,    0,   44,    0,   53,
        0,    0,    0,   21,    0,   47,   53,    0,    0,    0,
        0,    0,   29,    0,   45,    0,    0,    0,    0,   53,
       20,    0,    0,    0,   16,   22,   36,    0,    0,    0,
        0,   26,   46,    0,    0,    0,   49,    0,   45,    0,
        0,    0,    0,   35,    0,    0,    0,    0,   23,    0,
        0,    0,    0,    0,   46,   41,    0,    0,    0,   44,
        0,    0,    0,   47,    0,    0,    0,   48,   48,   45,

        0,    0,    0,    0,    0,   46,    0,    0,    0,   24,
        0,    0,   48,    0,   44,   44,   47,    0,   47,    0
    } ;

static yyconst int yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    2,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    4,    5,    6,    7,    8,    9,   10,   10,   10,
       10,   11,   10,   10,   10,   10,   10,   12,    1,    1,
       13,    1,    1,    1,   17,   18,   19,   20,   21,   22,
       23,   24,   25,   26,   26,   27,   28,   29,   30,   31,
       26,   32,   33,   34,   35,   36,   26,   37,   38,   26,
        1,    1,    1,   15,   16,    1,   17,   18,   19,   20,

       21,   22,   23,   24,   25,   26,   26,   27,   28,   29,
       30,   31,   26,   32,   33,   34,   35,   36,   26,   37,
       38,   26,   39,    1,   40,   41,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst int yy_meta[42] =
    {   0,
        1,    1,    1,    1,    1,    1,    2,    3,    1,    2,
        2,    3,    1,    4,    1,    5,    2,    2,    2,    2,
        2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
        2,    2,    2,    2,    2,    2,    2,    2,    1,    1,
        1
    } ;

static yyconst short int yy_base[334] =
    {   0,
        0,    0,  881,  882,   40,   42,  867,   39,  866,   45,
       49,   54,   56,   62,   69,   86,   89,   95,   98,   88,
      106,  109,   50,  115,  853,  104,  112,  882,  112,  113,
      141,  144,  149,  882,  882,  882,  152,  157,   66,  137,
      164,  170,  172,  178,  160,  169,  177,  182,  176,  188,
      211,  217,  198,  203,  212,  210,  219,  226,  228,  234,
      240,  246,  261,  267,  273,  275,  848,  882,  882,  850,
      278,  281,  285,  272,  277,  278,  280,  286,   51,  307,
      296,  295,  301,  304,  309,  310,  324,  325,  326,  338,
      340,  343,  353,  355,  358,  361,  129,  356,  375,  373,

      394,  396,  386,  387,  392,  399,  346,  407,  408,  840,
      313,  329,  415,  420,  423,  419,  447,  151,  440,  435,
      444,  453,  451,  458,  481,  852,  469,  474,  474,  482,
      493,  850,  830,  479,  490,  491,  496,  522,  524,  504,
      512,  532,  535,  407,  527,  532,  198,  551,  540,  542,
      547,  156,  565,  833,  836,  316,  829,  555,  560,  565,
      566,  578,  829,  572,  834,  819,  823,  573,  580,  587,
      588,  611,  600,  603,  335,  624,  613,  174,  882,  381,
      605,  622,  621,  817,  825,  440,  809,  821,  810,  831,
      623,  631,  628,  635,  653,  658,  418,  801,  641,  800,

      796,  764,  592,  668,  670,  671,  459,  682,  684,  745,
      732,  710,  711,  882,  688,  679,  533,  716,  643,  691,
      700,  677,  672,  698,  685,  692,  691,  711,  659,  697,
      648,  639,  617,  705,  213,  717,  712,  401,  618,  615,
      580,  671,  713,  700,  737,  580,  600,  565,  567,  721,
      728,  535,  531,  531,  882,  729,  882,  504,  491,  485,
      534,  737,  753,  468,  760,  458,  758,  764,  765,  468,
      767,  450,  768,  756,  745,  444,  444,  435,  882,  776,
      418,  370,  367,  775,  781,  882,  785,  336,  343,  788,
      283,  268,  251,  791,  244,  247,  224,  882,  797,  242,

      227,  200,  160,  115,  119,   79,  795,  802,   45,  882,
       39,  803,  801,  806,  808,   43,   36,  807,  809,  882,
      824,  827,  831,  835,  839,  843,  847,  851,  855,  859,
      863,  867,  871
    } ;

static yyconst short int yy_def[334] =
    {   0,
      320,    1,  320,  320,  320,  320,  320,  321,  320,  321,
      320,  321,  321,  321,  321,  321,  321,  321,  321,   17,
       17,   21,   21,   21,  320,  320,  320,  320,   21,  320,
      321,  320,  321,  320,  320,  320,  320,  321,   17,   17,
      321,  321,  321,  321,   17,   17,   17,   17,   21,   21,
      321,  321,   17,   17,   17,   17,   17,   21,   21,   21,
       21,   21,  321,  321,  321,  321,  320,  320,  320,  320,
      320,  320,  320,   21,   21,   21,   21,   21,  322,   41,
       21,   21,   21,   21,   21,   21,   21,   21,   21,   21,
       21,   21,   21,   21,   21,   21,  320,   21,   21,   21,

      321,  321,   17,   17,   21,   17,  323,   17,   17,  320,
      320,  320,   17,   17,   17,  322,  322,  324,  320,   17,
       17,   17,   17,   17,  321,  320,   21,   21,   17,   17,
       17,  320,  320,   17,   17,   17,   21,  321,  321,   17,
       17,  323,  323,  325,   17,   21,  320,  320,   17,   17,
       17,  320,  324,  320,  320,  320,  320,   17,   17,   21,
       21,  326,  320,   17,  320,  320,  320,   17,   17,   17,
       17,  327,   17,   21,  320,  325,   21,  320,  320,  320,
       21,   21,   21,  320,  320,  320,  320,  320,  320,  320,
       21,   21,   17,   17,  326,  326,  328,  320,   21,  320,

      320,  320,  321,  321,  327,  327,  329,  321,  321,  320,
      320,  320,  320,  320,  320,   21,  320,  320,  324,  320,
      320,  320,  320,  325,   21,   17,  320,  328,  320,   17,
      320,  320,  320,   17,  320,  329,   17,  320,  320,  320,
      320,  320,   17,  330,  320,  320,  320,  320,  320,   17,
       17,  320,  320,  320,  320,   17,  320,  320,  320,  320,
      320,   17,  320,  320,  331,  320,  320,  330,  330,  320,
      328,  320,  329,   17,  320,  320,  320,  320,  320,  320,
      320,  320,  320,  331,  331,  320,  320,  320,  320,  320,
      320,  320,  320,  320,  320,  320,  320,  320,  320,  320,

      320,  320,  320,  320,  320,  320,  320,  332,  320,  320,
      320,  333,  320,  332,  332,  320,  320,  333,  333,    0,
      320,  320,  320,  320,  320,  320,  320,  320,  320,  320,
      320,  320,  320
    } ;

static yyconst short int yy_nxt[924] =
    {   0,
        4,    5,    6,    4,    4,    4,    4,    7,    4,    8,
        8,    4,    9,   10,   11,    4,   12,   13,   14,   15,
       10,   16,   10,   10,   17,   10,   10,   18,   19,   20,
       21,   22,   23,   24,   10,   10,   10,   10,   25,    4,
       26,   27,   27,   27,   27,   29,   30,  235,   31,   31,
       32,   29,  320,   35,  227,   36,  320,  116,   37,   37,
       29,  320,   29,  320,   33,  320,  118,  320,   29,  320,
       64,  317,   38,  320,   33,   29,  320,  316,   43,   44,
      320,   33,   39,   65,   40,   75,   41,   42,   33,   46,
      175,   45,   29,  320,   76,   29,  320,  320,   33,   47,

      320,   29,  320,   77,   29,  320,  320,   68,   69,  320,
       33,   49,   48,   27,   27,   52,   33,   50,   54,   56,
       57,   51,   71,   71,   53,  312,   58,   55,   33,   61,
       59,  311,   33,   62,   33,   33,   33,   60,   33,   33,
       33,   63,   70,   33,   66,  132,   33,   29,   30,  133,
       31,   31,   32,   72,   72,   29,  320,  118,   73,   33,
      320,   37,   37,   29,  320,   33,  118,   78,  320,   33,
       79,  320,  184,   80,   80,  320,   29,  320,   29,  320,
      310,  320,   33,  320,   29,  320,  185,   84,   85,  320,
       74,   33,   86,   88,   81,   87,   33,   33,  213,   33,

       33,   33,   82,  178,   33,   33,   90,   91,   33,   33,
       33,   83,   33,  214,   33,   89,  309,   93,  320,   33,
       33,   92,  320,   29,  320,   33,   33,   97,  320,  260,
       95,   33,   33,  308,   33,   33,   94,  179,   33,   96,
       33,   33,   33,  261,   33,   98,  100,   33,   33,   99,
       33,   33,   33,  152,  102,  101,  306,   33,   33,   33,
       33,  105,  103,  104,   33,   33,   33,  107,  320,  305,
       33,   33,  320,   29,  320,  304,  106,   33,  320,   29,
      320,   29,  320,  303,  320,  111,  320,   71,   71,  109,
       72,   72,   33,  108,  112,  112,  113,   33,   33,  302,

       33,   33,   33,   33,   33,   83,  115,  114,   33,   33,
       33,   33,  120,   29,  301,   33,   33,   33,  119,   33,
       33,   33,  148,  148,   33,   33,   33,   33,   33,   33,
       33,  121,   33,   33,   33,   33,  188,   33,  112,  112,
       33,   33,  122,  123,  124,   33,   33,  189,   33,   33,
       33,  210,  142,  126,  119,   33,   33,  125,   33,   33,
       33,  144,   33,  127,   33,  211,  212,   33,  300,   33,
       83,   33,  128,   33,   33,   33,   33,  129,   33,   33,
       33,   33,   33,  297,   33,   33,   33,   33,  130,   33,
      215,  215,   33,   33,  131,   33,   83,   33,  296,   33,

      135,  320,   29,  320,  134,  320,   33,  320,  138,   33,
      263,  263,   33,  144,   33,   33,  140,  136,   33,   33,
      139,   33,  144,   33,  197,  116,  137,  141,  145,   33,
       33,   33,   33,  197,  118,   33,   33,   33,  295,   33,
       33,  146,   33,   33,  149,   33,  219,   33,   33,  220,
      220,  151,   33,  116,  150,   33,  154,   33,  152,  293,
       33,  155,  118,   33,  292,  207,   33,   33,  158,  160,
      156,  157,   33,   33,  207,   33,   33,  291,  159,   33,
       33,   33,  289,   33,  288,   33,   33,  162,  320,   33,
       33,  161,  320,   33,   33,  168,   33,  286,   33,  283,

      164,   33,   33,   83,   33,   33,   83,   33,  165,  169,
       33,   33,   33,   33,   33,   33,  171,  280,   33,   33,
       33,   33,  170,   33,  279,   33,   33,   33,  172,  320,
       29,  320,  174,  320,   83,  320,   33,  278,  142,  244,
       33,  142,  245,  245,   33,   33,  175,  144,   33,   33,
      144,  277,   33,  173,  281,   33,  177,  276,  180,   33,
      148,  148,   33,   33,   33,  282,  183,  275,  181,   33,
      182,  118,   33,  273,   33,   33,  152,   33,  191,   33,
      118,  272,   33,   33,  195,   33,  194,   33,  192,  193,
       33,   33,   33,  197,   33,   33,   33,   33,   29,  320,

      199,   33,   33,  119,   33,  203,  271,   83,   33,   33,
       33,  270,   33,  266,   83,   33,  204,  205,  208,   33,
       33,  265,   33,  209,   33,   33,  207,   33,   33,   33,
      144,   83,   33,   33,   33,  175,   33,   33,  264,  144,
      216,   33,   33,   33,   33,   33,   33,   33,  259,  118,
       33,   33,   33,   33,   33,  225,  226,   33,  118,  195,
       33,  230,   33,   33,  195,   33,   33,   33,  197,  227,
      258,   33,   33,  197,   29,  320,  205,  205,  257,  320,
      267,  267,  235,   33,   33,  207,  207,  234,   29,  320,
       29,  320,  255,  320,  249,  320,  242,  215,  215,  243,

      220,  220,  119,   33,  144,   33,  268,  252,  248,   33,
       33,  237,  253,  144,   33,  268,   33,  197,  250,   33,
       33,  254,  227,  207,  251,   33,  197,   33,  235,   33,
      256,  247,  207,   33,   33,   33,  246,   33,  262,  241,
       33,   33,  240,   33,   33,   33,  245,  245,  152,   33,
       33,   33,  239,   33,  290,  290,   33,   33,  274,   33,
       33,   33,  263,  263,  175,   33,  284,  267,  267,   33,
      268,  268,  287,  197,  207,  284,  152,  238,   33,  268,
      268,  284,  197,  207,   33,  294,  294,  284,   33,  298,
      284,  298,  175,  233,  299,  299,  284,  290,  290,  227,

      294,  294,  235,  307,  313,  313,  299,  299,  314,  318,
      313,  313,  314,  318,  314,  318,  232,  314,  318,  227,
      235,  314,  318,  314,  318,   33,   33,   33,  117,  231,
      117,  117,  143,  229,  143,  143,  153,  224,  223,  153,
      176,  222,  221,  176,  196,  218,  196,  196,  206,  217,
      206,  206,  228,  202,  201,  228,  236,  200,  198,  236,
      269,  190,  187,  269,  285,  186,  167,  285,  315,  166,
      163,  315,  319,  147,   67,  319,  110,   67,   34,   28,
      320,    3,  320,  320,  320,  320,  320,  320,  320,  320,
      320,  320,  320,  320,  320,  320,  320,  320,  320,  320,

      320,  320,  320,  320,  320,  320,  320,  320,  320,  320,
      320,  320,  320,  320,  320,  320,  320,  320,  320,  320,
      320,  320,  320
    } ;

static yyconst short int yy_chk[924] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    5,    5,    6,    6,    8,    8,  317,    8,    8,
        8,   10,   10,   11,  316,   11,   10,   79,   11,   11,
       12,   12,   13,   13,   79,   12,   79,   13,   14,   14,
       23,  311,   12,   14,   23,   15,   15,  309,   13,   14,
       15,   23,   12,   23,   12,   39,   12,   12,   39,   15,
      306,   14,   16,   16,   39,   17,   17,   16,   39,   15,

       17,   18,   18,   39,   19,   19,   18,   26,   26,   19,
       20,   17,   16,   27,   27,   18,   20,   17,   19,   20,
       20,   17,   30,   30,   18,  305,   21,   19,   21,   22,
       21,  304,   29,   22,   21,   24,   29,   21,   21,   24,
       22,   22,   26,   29,   24,   97,   24,   31,   31,   97,
       31,   31,   31,   32,   32,   33,   33,  118,   37,   40,
       33,   37,   37,   38,   38,   40,  118,   40,   38,   40,
       41,   41,  152,   41,   41,   41,   42,   42,   43,   43,
      303,   42,   45,   43,   44,   44,  152,   45,   45,   44,
       38,   46,   45,   47,   41,   46,   49,   46,  178,   47,

       49,   46,   43,  147,   48,   47,   49,   49,   50,   47,
       48,   44,   50,  178,   48,   48,  302,   51,   51,   50,
       53,   50,   51,   52,   52,   54,   53,   55,   52,  235,
       53,   54,   56,  301,   55,   54,   52,  147,   56,   54,
       55,   57,   56,  235,   55,   55,   58,   57,   59,   57,
       58,   57,   59,  300,   60,   59,  297,   58,   60,   59,
       61,   61,   60,   60,   61,   60,   62,   63,   63,  296,
       62,   61,   63,   64,   64,  295,   62,   62,   64,   65,
       65,   66,   66,  293,   65,   71,   66,   71,   71,   65,
       72,   72,   74,   64,   73,   73,   74,   75,   76,  292,

       77,   75,   76,   74,   77,   82,   78,   76,   75,   76,
       78,   77,   81,   80,  291,   82,   81,   78,   80,   82,
       81,   83,  111,  111,   84,   83,   82,   81,   84,   85,
       86,   84,   83,   85,   86,   84,  156,   80,  112,  112,
       85,   86,   85,   86,   87,   88,   89,  156,   87,   88,
       89,  175,  107,   90,  289,   87,   88,   89,   90,  107,
       91,  107,   90,   92,   91,  175,  175,   92,  288,   90,
       91,   91,   92,   93,   92,   94,   98,   93,   95,   94,
       98,   96,   95,  283,   93,   96,   94,   98,   95,   95,
      180,  180,   96,  100,   96,   99,   99,  100,  282,   99,

      101,  101,  102,  102,  100,  101,   99,  102,  103,  104,
      238,  238,  105,  144,  103,  104,  105,  102,  103,  104,
      104,  106,  144,  105,  197,  116,  102,  106,  108,  108,
      109,  106,  116,  197,  116,  108,  109,  113,  281,  108,
      109,  109,  114,  113,  113,  115,  186,  113,  114,  186,
      186,  115,  114,  117,  114,  115,  119,  120,  117,  278,
      117,  119,  117,  120,  277,  207,  121,  120,  120,  122,
      119,  119,  121,  123,  207,  122,  121,  276,  121,  123,
      124,  122,  272,  123,  270,  122,  124,  125,  125,  127,
      124,  124,  125,  127,  128,  134,  129,  266,  128,  264,

      127,  134,  129,  130,  130,  128,  129,  134,  131,  135,
      130,  134,  135,  136,  130,  131,  137,  260,  135,  136,
      137,  131,  135,  136,  259,  131,  140,  137,  138,  138,
      139,  139,  140,  138,  141,  139,  140,  258,  142,  217,
      141,  143,  217,  217,  141,  142,  143,  142,  143,  145,
      143,  254,  146,  139,  261,  145,  146,  253,  148,  145,
      148,  148,  149,  146,  150,  261,  151,  252,  149,  151,
      150,  153,  149,  249,  150,  151,  153,  158,  158,  151,
      153,  248,  159,  158,  162,  160,  161,  158,  159,  160,
      161,  162,  159,  162,  164,  168,  160,  161,  203,  203,

      164,  168,  169,  203,  164,  168,  247,  169,  169,  170,
      171,  246,  169,  241,  170,  170,  171,  172,  173,  170,
      171,  240,  173,  174,  172,  181,  172,  174,  173,  181,
      176,  177,  173,  177,  174,  176,  181,  177,  239,  176,
      182,  183,  182,  191,  177,  183,  182,  191,  233,  219,
      193,  192,  183,  182,  191,  192,  193,  194,  219,  195,
      193,  199,  192,  194,  196,  199,  195,  194,  195,  196,
      232,  196,  199,  196,  204,  204,  205,  206,  231,  204,
      242,  242,  206,  205,  206,  205,  206,  204,  208,  208,
      209,  209,  229,  208,  223,  209,  215,  215,  215,  216,

      220,  220,  220,  216,  224,  225,  244,  227,  222,  225,
      216,  208,  227,  224,  226,  244,  225,  228,  225,  230,
      226,  227,  228,  236,  226,  230,  228,  234,  236,  230,
      230,  221,  236,  234,  237,  243,  218,  234,  237,  213,
      237,  243,  212,  250,  237,  243,  245,  245,  245,  250,
      251,  256,  211,  250,  275,  275,  251,  256,  250,  262,
      251,  256,  263,  263,  263,  262,  265,  267,  267,  262,
      268,  269,  267,  271,  273,  265,  269,  210,  274,  268,
      269,  284,  271,  273,  274,  280,  280,  285,  274,  287,
      284,  287,  285,  202,  287,  287,  285,  290,  290,  290,

      294,  294,  294,  299,  307,  307,  299,  299,  308,  312,
      313,  313,  314,  318,  315,  319,  201,  308,  312,  315,
      319,  314,  318,  315,  319,  321,  321,  321,  322,  200,
      322,  322,  323,  198,  323,  323,  324,  190,  189,  324,
      325,  188,  187,  325,  326,  185,  326,  326,  327,  184,
      327,  327,  328,  167,  166,  328,  329,  165,  163,  329,
      330,  157,  155,  330,  331,  154,  133,  331,  332,  132,
      126,  332,  333,  110,   70,  333,   67,   25,    9,    7,
        3,  320,  320,  320,  320,  320,  320,  320,  320,  320,
      320,  320,  320,  320,  320,  320,  320,  320,  320,  320,

      320,  320,  320,  320,  320,  320,  320,  320,  320,  320,
      320,  320,  320,  320,  320,  320,  320,  320,  320,  320,
      320,  320,  320
    } ;

static yy_state_type yy_last_accepting_state;
static char *yy_last_accepting_cpos;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
char *yytext;
#line 1 "export.l"
#define INITIAL 0
#line 2 "export.l"
/*
  filename: export.l

  description:
    Defines the tokenizer for an RPSL export attribute.  It was mostly
    stolen from the IRRToolSet, simplified by removing ability to parse
    things defined by a dictionary (we use XML for extensibility rather
    than a dictionary).

  notes:
    Tokens are defined in the associated grammar, export.y.

  $Id: export.lex.c,v 1.2 2002/02/20 16:41:48 shane Exp $
*/
#line 33 "export.l"
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

/* tokens defined in the grammar */
#include "export.tab.h"

#define exportwrap yywrap
void syntax_error(char *fmt, ...);
void yy_input(char *buf, int *result, int max_size);
#undef YY_INPUT
#define YY_INPUT(buf,result,max_size) yy_input(buf,&result,max_size)
#line 722 "export.lex.c"

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap YY_PROTO(( void ));
#else
extern int yywrap YY_PROTO(( void ));
#endif
#endif

#ifndef YY_NO_UNPUT
static void yyunput YY_PROTO(( int c, char *buf_ptr ));
#endif

#ifndef yytext_ptr
static void yy_flex_strncpy YY_PROTO(( char *, yyconst char *, int ));
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen YY_PROTO(( yyconst char * ));
#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int yyinput YY_PROTO(( void ));
#else
static int input YY_PROTO(( void ));
#endif
#endif

#if YY_STACK_USED
static int yy_start_stack_ptr = 0;
static int yy_start_stack_depth = 0;
static int *yy_start_stack = 0;
#ifndef YY_NO_PUSH_STATE
static void yy_push_state YY_PROTO(( int new_state ));
#endif
#ifndef YY_NO_POP_STATE
static void yy_pop_state YY_PROTO(( void ));
#endif
#ifndef YY_NO_TOP_STATE
static int yy_top_state YY_PROTO(( void ));
#endif

#else
#define YY_NO_PUSH_STATE 1
#define YY_NO_POP_STATE 1
#define YY_NO_TOP_STATE 1
#endif

#ifdef YY_MALLOC_DECL
YY_MALLOC_DECL
#else
#if __STDC__
#ifndef __cplusplus
#include <stdlib.h>
#endif
#else
/* Just try to get by without declaring the routines.  This will fail
 * miserably on non-ANSI systems for which sizeof(size_t) != sizeof(int)
 * or sizeof(void*) != sizeof(int).
 */
#endif
#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */

#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO (void) fwrite( yytext, yyleng, 1, yyout )
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( yy_current_buffer->yy_is_interactive ) \
		{ \
		int c = '*', n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else if ( ((result = fread( buf, 1, max_size, yyin )) == 0) \
		  && ferror( yyin ) ) \
		YY_FATAL_ERROR( "input in flex scanner failed" );
#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg )
#endif

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL int yylex YY_PROTO(( void ))
#endif

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

#define YY_RULE_SETUP \
	YY_USER_ACTION

YY_DECL
	{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;

#line 47 "export.l"


#line 876 "export.lex.c"

	if ( yy_init )
		{
		yy_init = 0;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! yy_start )
			yy_start = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! yy_current_buffer )
			yy_current_buffer =
				yy_create_buffer( yyin, YY_BUF_SIZE );

		yy_load_buffer_state();
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
		yy_cp = yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = yy_start;
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				yy_last_accepting_state = yy_current_state;
				yy_last_accepting_cpos = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 321 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 882 );

yy_find_action:
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;


do_action:	/* This label is used only to access EOF actions. */


		switch ( yy_act )
	{ /* beginning of action switch */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = yy_hold_char;
			yy_cp = yy_last_accepting_cpos;
			yy_current_state = yy_last_accepting_state;
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 49 "export.l"
{ ; }
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 51 "export.l"
{ return OP_OR; }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 52 "export.l"
{ return OP_AND; }
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 53 "export.l"
{ return OP_NOT; }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 54 "export.l"
{ return OP_COMPARE; }
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 55 "export.l"
{ return OP_EQUAL; }
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 56 "export.l"
{ return OP_APPEND; }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 58 "export.l"
{ return OP_MS; }
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 59 "export.l"
{ return OP_MS; }
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 60 "export.l"
{ 
    long int val;
    val = strtol(yytext+1, NULL, 10);
    if ((val < 0) || (val > 32)) {
        syntax_error("more specifics operator \"%s\" not 0 to 32 bits", 
                     yytext);
    }
    return OP_MS;
}
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 69 "export.l"
{
    long int val1, val2;
    char *p;
    val1 = strtol(yytext+1, &p, 10);
    if ((val1 < 0) || (val1 > 32)) {
        syntax_error("more specifics operator \"%s\" not 0 to 32 bits", 
                     yytext);
    }
    val2 = strtol(p+1, NULL, 10);
    if ((val2 < 0) || (val2 > 32)) {
        syntax_error("more specifics operator \"%s\" not 0 to 32 bits", 
                     yytext);
    }
    if (val2 < val1) {
        syntax_error("more specifics operator \"%s\" not 0 to 32 bits", 
                     yytext);
    }
    return OP_MS;
}
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 90 "export.l"
{ return KEYW_ANY; }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 91 "export.l"
{ return KEYW_PEERAS; }
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 93 "export.l"
{ return KEYW_TO; }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 94 "export.l"
{ return KEYW_ACTION; }
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 95 "export.l"
{ return KEYW_IGP_COST; }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 96 "export.l"
{ return KEYW_SELF; }
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 97 "export.l"
{ return KEYW_APPEND; }
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 98 "export.l"
{ return KEYW_DELETE; }
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 99 "export.l"
{ return KEYW_CONTAINS; }
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 100 "export.l"
{ return KEYW_PREPEND; }
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 102 "export.l"
{ return KEYW_INTERNET; }
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 103 "export.l"
{ return KEYW_NO_EXPORT; }
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 104 "export.l"
{ return KEYW_NO_ADVERTISE; }
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 106 "export.l"
{ return KEYW_AT; }
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 107 "export.l"
{ return KEYW_PROTOCOL; }
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 108 "export.l"
{ return KEYW_INTO; }
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 109 "export.l"
{ return KEYW_REFINE; }
	YY_BREAK
case 29:
YY_RULE_SETUP
#line 110 "export.l"
{ return KEYW_ANNOUNCE; }
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 112 "export.l"
{
    return TKN_PROTOCOL;
}
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 116 "export.l"
{ return TKN_PREF; }
	YY_BREAK
case 32:
YY_RULE_SETUP
#line 117 "export.l"
{ return TKN_MED; }
	YY_BREAK
case 33:
YY_RULE_SETUP
#line 118 "export.l"
{ return TKN_DPA; }
	YY_BREAK
case 34:
YY_RULE_SETUP
#line 119 "export.l"
{ return TKN_ASPATH; }
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 120 "export.l"
{ return TKN_COMMUNITY; }
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 121 "export.l"
{ return TKN_NEXT_HOP; }
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 122 "export.l"
{ return TKN_COST; }
	YY_BREAK
case 38:
YY_RULE_SETUP
#line 124 "export.l"
{ return ASPATH_POSTFIX; }
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 125 "export.l"
{ return ASPATH_POSTFIX; }
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 126 "export.l"
{ return ASPATH_POSTFIX; }
	YY_BREAK
case 41:
YY_RULE_SETUP
#line 127 "export.l"
{ return ASPATH_POSTFIX; }
	YY_BREAK
case 42:
YY_RULE_SETUP
#line 128 "export.l"
{ return ASPATH_POSTFIX; }
	YY_BREAK
case 43:
YY_RULE_SETUP
#line 130 "export.l"
{
    long int val;
    char *s;
    val = strtol(yytext+2, &s, 10);
    if ((val < 0) || (val > 65535) || (*s != '\0')) {
        syntax_error("AS number \"%s\" is not between 0 and 65535", yytext);
    }
    return TKN_ASNO;
}
	YY_BREAK
case 44:
YY_RULE_SETUP
#line 140 "export.l"
{
    return TKN_FLTRNAME;
}
	YY_BREAK
case 45:
YY_RULE_SETUP
#line 144 "export.l"
{
    return TKN_ASNAME;
}
	YY_BREAK
case 46:
YY_RULE_SETUP
#line 148 "export.l"
{
    return TKN_RSNAME;
}
	YY_BREAK
case 47:
YY_RULE_SETUP
#line 152 "export.l"
{
    return TKN_PRNGNAME;
}
	YY_BREAK
case 48:
YY_RULE_SETUP
#line 156 "export.l"
{
    /* check each number of 1.2.3.4/5 in prefix is valid, as
       well as any bit ranges specified */
    long int val;
    long int endval;
    char *s, *p;
    p = s = yytext;
    while (*s != '/') {
        val = strtol(p, &s, 10);
        if ((val < 0) || (val > 255)) {
             syntax_error("IP prefix \"%s\" contains an invalid octet", 
                          yytext);
        }
        p = s + 1;
    }
    val = strtol(p, &s, 10);
    if ((val < 0) || (val > 32)) {
        syntax_error("IP prefix range \"%s\" contains an invalid prefix length",
                     yytext);
    }
    p = s + 1;
    if (isdigit((int)*p)) {
        val = strtol(p, &s, 10);
        if ((val < 0) || (val > 32)) {
            syntax_error("IP prefix range \"%s\" contains an invalid range",
                         yytext);
        }
        p = s + 1;
        if (isdigit((int)*p)) {
            endval = strtol(p, NULL, 10);
            if ((endval < 0) || (endval > 32)) {
                syntax_error("IP prefix \"%s\" contains an invalid prefix range", 
                             yytext);
            }
            if (endval < val) {
                syntax_error("IP prefix \"%s\" range end is less than range start",
                             yytext);
            }
        }
    }
    return TKN_PRFXV4RNG;
}
	YY_BREAK
case 49:
YY_RULE_SETUP
#line 199 "export.l"
{
    /* check each number of 1.2.3.4/5 in prefix is valid */
    long int val;
    char *s, *p;
    p = s = yytext;
    while (*s != '/') {
        val = strtol(p, &s, 10);
        if ((val < 0) || (val > 255)) {
             syntax_error("IP prefix \"%s\" contains an invalid octet", 
                          yytext);
        }
        p = s + 1;
    }
    val = strtol(p, NULL, 10);
    if ((val < 0) || (val > 32)) {
        syntax_error("IP prefix \"%s\" contains an invalid prefix length", 
                     yytext);
    }
    return TKN_PRFXV4;
}
	YY_BREAK
case 50:
YY_RULE_SETUP
#line 220 "export.l"
{
    /* check each number of 1.2.3.4 is valid */
    long int val;
    char *s, *p;
    p = s = yytext;
    while (*s != '\0') {
        val = strtol(p, &s, 10);
        if ((val < 0) || (val > 255)) {
             syntax_error("IP address \"%s\" contains an invalid octet", 
                          yytext);
        }
        p = s + 1;
    }
    return TKN_IPV4;
}
	YY_BREAK
case 51:
YY_RULE_SETUP
#line 236 "export.l"
{
    /* verify each part is a 16-bit number */
    long int val;
    char *s, *p;
    p = s = yytext;
    while (*s != '\0') {
        val = strtol(p, &s, 10);
        if ((val < 0) || (val > 65535)) {
             syntax_error("Community number \"%s\" contains an invalid number", 
                           yytext);
        }
        p = s + 1;
    }
    return TKN_COMM_NO;
}
	YY_BREAK
case 52:
YY_RULE_SETUP
#line 252 "export.l"
{ 
    exportlval.sval = yytext;
    return TKN_INT; 
}
	YY_BREAK
case 53:
YY_RULE_SETUP
#line 257 "export.l"
{ 
    /* check the length */
    if (strlen(yytext) > 63) {
        syntax_error("Portion of domain name \"%s\" is longer "
                                                          "than 63 characters", 
                     yytext);
    }
    exportlval.sval = yytext;
    return TKN_DNS; 
}
	YY_BREAK
case 54:
YY_RULE_SETUP
#line 268 "export.l"
{ return yytext[0]; }
	YY_BREAK
case 55:
YY_RULE_SETUP
#line 270 "export.l"
ECHO;
	YY_BREAK
#line 1378 "export.lex.c"
case YY_STATE_EOF(INITIAL):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - yytext_ptr) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yy_hold_char;
		YY_RESTORE_YY_MORE_OFFSET

		if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between yy_current_buffer and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yy_n_chars = yy_current_buffer->yy_n_chars;
			yy_current_buffer->yy_input_file = yyin;
			yy_current_buffer->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yy_c_buf_p <= &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yy_c_buf_p = yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state();

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state );

			yy_bp = yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = yy_c_buf_p;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer() )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yy_did_buffer_switch_on_eof = 0;

				if ( yywrap() )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yy_c_buf_p = yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yy_c_buf_p =
					yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yy_c_buf_p =
				&yy_current_buffer->yy_ch_buf[yy_n_chars];

				yy_current_state = yy_get_previous_state();

				yy_cp = yy_c_buf_p;
				yy_bp = yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of yylex */


/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */

static int yy_get_next_buffer()
	{
	register char *dest = yy_current_buffer->yy_ch_buf;
	register char *source = yytext_ptr;
	register int number_to_move, i;
	int ret_val;

	if ( yy_c_buf_p > &yy_current_buffer->yy_ch_buf[yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( yy_current_buffer->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yy_c_buf_p - yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) (yy_c_buf_p - yytext_ptr) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( yy_current_buffer->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		yy_current_buffer->yy_n_chars = yy_n_chars = 0;

	else
		{
		int num_to_read =
			yy_current_buffer->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */
#ifdef YY_USES_REJECT
			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );
#else

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = yy_current_buffer;

			int yy_c_buf_p_offset =
				(int) (yy_c_buf_p - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					yy_flex_realloc( (void *) b->yy_ch_buf,
							 b->yy_buf_size + 2 );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = yy_current_buffer->yy_buf_size -
						number_to_move - 1;
#endif
			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&yy_current_buffer->yy_ch_buf[number_to_move]),
			yy_n_chars, num_to_read );

		yy_current_buffer->yy_n_chars = yy_n_chars;
		}

	if ( yy_n_chars == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin );
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			yy_current_buffer->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	yy_n_chars += number_to_move;
	yy_current_buffer->yy_ch_buf[yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	yy_current_buffer->yy_ch_buf[yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	yytext_ptr = &yy_current_buffer->yy_ch_buf[0];

	return ret_val;
	}


/* yy_get_previous_state - get the state just before the EOB char was reached */

static yy_state_type yy_get_previous_state()
	{
	register yy_state_type yy_current_state;
	register char *yy_cp;

	yy_current_state = yy_start;

	for ( yy_cp = yytext_ptr + YY_MORE_ADJ; yy_cp < yy_c_buf_p; ++yy_cp )
		{
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			yy_last_accepting_state = yy_current_state;
			yy_last_accepting_cpos = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 321 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
	}


/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */

#ifdef YY_USE_PROTOS
static yy_state_type yy_try_NUL_trans( yy_state_type yy_current_state )
#else
static yy_state_type yy_try_NUL_trans( yy_current_state )
yy_state_type yy_current_state;
#endif
	{
	register int yy_is_jam;
	register char *yy_cp = yy_c_buf_p;

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		yy_last_accepting_state = yy_current_state;
		yy_last_accepting_cpos = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 321 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 320);

	return yy_is_jam ? 0 : yy_current_state;
	}


#ifndef YY_NO_UNPUT
#ifdef YY_USE_PROTOS
static void yyunput( int c, register char *yy_bp )
#else
static void yyunput( c, yy_bp )
int c;
register char *yy_bp;
#endif
	{
	register char *yy_cp = yy_c_buf_p;

	/* undo effects of setting up yytext */
	*yy_cp = yy_hold_char;

	if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		register int number_to_move = yy_n_chars + 2;
		register char *dest = &yy_current_buffer->yy_ch_buf[
					yy_current_buffer->yy_buf_size + 2];
		register char *source =
				&yy_current_buffer->yy_ch_buf[number_to_move];

		while ( source > yy_current_buffer->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		yy_current_buffer->yy_n_chars =
			yy_n_chars = yy_current_buffer->yy_buf_size;

		if ( yy_cp < yy_current_buffer->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;


	yytext_ptr = yy_bp;
	yy_hold_char = *yy_cp;
	yy_c_buf_p = yy_cp;
	}
#endif	/* ifndef YY_NO_UNPUT */


#ifdef __cplusplus
static int yyinput()
#else
static int input()
#endif
	{
	int c;

	*yy_c_buf_p = yy_hold_char;

	if ( *yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yy_c_buf_p < &yy_current_buffer->yy_ch_buf[yy_n_chars] )
			/* This was really a NUL. */
			*yy_c_buf_p = '\0';

		else
			{ /* need more input */
			int offset = yy_c_buf_p - yytext_ptr;
			++yy_c_buf_p;

			switch ( yy_get_next_buffer() )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					yyrestart( yyin );

					/* fall through */

				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap() )
						return EOF;

					if ( ! yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput();
#else
					return input();
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					yy_c_buf_p = yytext_ptr + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) yy_c_buf_p;	/* cast for 8-bit char's */
	*yy_c_buf_p = '\0';	/* preserve yytext */
	yy_hold_char = *++yy_c_buf_p;


	return c;
	}


#ifdef YY_USE_PROTOS
void yyrestart( FILE *input_file )
#else
void yyrestart( input_file )
FILE *input_file;
#endif
	{
	if ( ! yy_current_buffer )
		yy_current_buffer = yy_create_buffer( yyin, YY_BUF_SIZE );

	yy_init_buffer( yy_current_buffer, input_file );
	yy_load_buffer_state();
	}


#ifdef YY_USE_PROTOS
void yy_switch_to_buffer( YY_BUFFER_STATE new_buffer )
#else
void yy_switch_to_buffer( new_buffer )
YY_BUFFER_STATE new_buffer;
#endif
	{
	if ( yy_current_buffer == new_buffer )
		return;

	if ( yy_current_buffer )
		{
		/* Flush out information for old buffer. */
		*yy_c_buf_p = yy_hold_char;
		yy_current_buffer->yy_buf_pos = yy_c_buf_p;
		yy_current_buffer->yy_n_chars = yy_n_chars;
		}

	yy_current_buffer = new_buffer;
	yy_load_buffer_state();

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yy_did_buffer_switch_on_eof = 1;
	}


#ifdef YY_USE_PROTOS
void yy_load_buffer_state( void )
#else
void yy_load_buffer_state()
#endif
	{
	yy_n_chars = yy_current_buffer->yy_n_chars;
	yytext_ptr = yy_c_buf_p = yy_current_buffer->yy_buf_pos;
	yyin = yy_current_buffer->yy_input_file;
	yy_hold_char = *yy_c_buf_p;
	}


#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_create_buffer( FILE *file, int size )
#else
YY_BUFFER_STATE yy_create_buffer( file, size )
FILE *file;
int size;
#endif
	{
	YY_BUFFER_STATE b;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yy_flex_alloc( b->yy_buf_size + 2 );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer( b, file );

	return b;
	}


#ifdef YY_USE_PROTOS
void yy_delete_buffer( YY_BUFFER_STATE b )
#else
void yy_delete_buffer( b )
YY_BUFFER_STATE b;
#endif
	{
	if ( ! b )
		return;

	if ( b == yy_current_buffer )
		yy_current_buffer = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yy_flex_free( (void *) b->yy_ch_buf );

	yy_flex_free( (void *) b );
	}


#ifndef YY_ALWAYS_INTERACTIVE
#ifndef YY_NEVER_INTERACTIVE
extern int isatty YY_PROTO(( int ));
#endif
#endif

#ifdef YY_USE_PROTOS
void yy_init_buffer( YY_BUFFER_STATE b, FILE *file )
#else
void yy_init_buffer( b, file )
YY_BUFFER_STATE b;
FILE *file;
#endif


	{
	yy_flush_buffer( b );

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

#if YY_ALWAYS_INTERACTIVE
	b->yy_is_interactive = 1;
#else
#if YY_NEVER_INTERACTIVE
	b->yy_is_interactive = 0;
#else
	b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
#endif
#endif
	}


#ifdef YY_USE_PROTOS
void yy_flush_buffer( YY_BUFFER_STATE b )
#else
void yy_flush_buffer( b )
YY_BUFFER_STATE b;
#endif

	{
	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == yy_current_buffer )
		yy_load_buffer_state();
	}


#ifndef YY_NO_SCAN_BUFFER
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_buffer( char *base, yy_size_t size )
#else
YY_BUFFER_STATE yy_scan_buffer( base, size )
char *base;
yy_size_t size;
#endif
	{
	YY_BUFFER_STATE b;

	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) yy_flex_alloc( sizeof( struct yy_buffer_state ) );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer( b );

	return b;
	}
#endif


#ifndef YY_NO_SCAN_STRING
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_string( yyconst char *yy_str )
#else
YY_BUFFER_STATE yy_scan_string( yy_str )
yyconst char *yy_str;
#endif
	{
	int len;
	for ( len = 0; yy_str[len]; ++len )
		;

	return yy_scan_bytes( yy_str, len );
	}
#endif


#ifndef YY_NO_SCAN_BYTES
#ifdef YY_USE_PROTOS
YY_BUFFER_STATE yy_scan_bytes( yyconst char *bytes, int len )
#else
YY_BUFFER_STATE yy_scan_bytes( bytes, len )
yyconst char *bytes;
int len;
#endif
	{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;

	/* Get memory for full buffer, including space for trailing EOB's. */
	n = len + 2;
	buf = (char *) yy_flex_alloc( n );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < len; ++i )
		buf[i] = bytes[i];

	buf[len] = buf[len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer( buf, n );
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
	}
#endif


#ifndef YY_NO_PUSH_STATE
#ifdef YY_USE_PROTOS
static void yy_push_state( int new_state )
#else
static void yy_push_state( new_state )
int new_state;
#endif
	{
	if ( yy_start_stack_ptr >= yy_start_stack_depth )
		{
		yy_size_t new_size;

		yy_start_stack_depth += YY_START_STACK_INCR;
		new_size = yy_start_stack_depth * sizeof( int );

		if ( ! yy_start_stack )
			yy_start_stack = (int *) yy_flex_alloc( new_size );

		else
			yy_start_stack = (int *) yy_flex_realloc(
					(void *) yy_start_stack, new_size );

		if ( ! yy_start_stack )
			YY_FATAL_ERROR(
			"out of memory expanding start-condition stack" );
		}

	yy_start_stack[yy_start_stack_ptr++] = YY_START;

	BEGIN(new_state);
	}
#endif


#ifndef YY_NO_POP_STATE
static void yy_pop_state()
	{
	if ( --yy_start_stack_ptr < 0 )
		YY_FATAL_ERROR( "start-condition stack underflow" );

	BEGIN(yy_start_stack[yy_start_stack_ptr]);
	}
#endif


#ifndef YY_NO_TOP_STATE
static int yy_top_state()
	{
	return yy_start_stack[yy_start_stack_ptr - 1];
	}
#endif

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

#ifdef YY_USE_PROTOS
static void yy_fatal_error( yyconst char msg[] )
#else
static void yy_fatal_error( msg )
char msg[];
#endif
	{
	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
	}



/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
		yytext[yyleng] = yy_hold_char; \
		yy_c_buf_p = yytext + n; \
		yy_hold_char = *yy_c_buf_p; \
		*yy_c_buf_p = '\0'; \
		yyleng = n; \
		} \
	while ( 0 )


/* Internal utility routines. */

#ifndef yytext_ptr
#ifdef YY_USE_PROTOS
static void yy_flex_strncpy( char *s1, yyconst char *s2, int n )
#else
static void yy_flex_strncpy( s1, s2, n )
char *s1;
yyconst char *s2;
int n;
#endif
	{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
	}
#endif

#ifdef YY_NEED_STRLEN
#ifdef YY_USE_PROTOS
static int yy_flex_strlen( yyconst char *s )
#else
static int yy_flex_strlen( s )
yyconst char *s;
#endif
	{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
	}
#endif


#ifdef YY_USE_PROTOS
static void *yy_flex_alloc( yy_size_t size )
#else
static void *yy_flex_alloc( size )
yy_size_t size;
#endif
	{
	return (void *) malloc( size );
	}

#ifdef YY_USE_PROTOS
static void *yy_flex_realloc( void *ptr, yy_size_t size )
#else
static void *yy_flex_realloc( ptr, size )
void *ptr;
yy_size_t size;
#endif
	{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
	}

#ifdef YY_USE_PROTOS
static void yy_flex_free( void *ptr )
#else
static void yy_flex_free( ptr )
void *ptr;
#endif
	{
	free( ptr );
	}

#if YY_MAIN
int main()
	{
	yylex();
	return 0;
	}
#endif
#line 270 "export.l"


void 
export_reset ()
{
    yy_flush_buffer(YY_CURRENT_BUFFER);
}

