char * const Attribute_aliases[] = {
  "ad",
  "address",
  "ac",
  "admin-c",
  "ag",
  "aggr-mtd",
  "ab",
  "aggr-bndry",
  "az",
  "alias",
  "aa",
  "as-name",
  "as",
  "as-set",
  "ch",
  "changed",
  "co",
  "components",
  "df",
  "default",
  "de",
  "descr",
  "em",
  "e-mail",
  "ex",
  "export",
  "ec",
  "export-comps",
  "fx",
  "fax-no",
  "fi",
  "filter",
  "fs",
  "filter-set",
  "ho",
  "holes",
  "if",
  "ifaddr",
  "ip",
  "import",
  "ij",
  "inject",
  "mr",
  "mbrs-by-ref",
  "ms",
  "members",
  "mb",
  "mnt-by",
  "mn",
  "mnt-nfy",
  "mt",
  "mntner",
  "nh",
  "nic-hdl",
  "ny",
  "notify",
  "or",
  "origin",
  "pe",
  "peer",
  "pg",
  "peering",
  "pn",
  "person",
  "ph",
  "phone",
  "rm",
  "remarks",
  "ro",
  "role",
  "rs",
  "route-set",
  "so",
  "source",
  "tc",
  "tech-c",
  "tb",
  "trouble",
  "dt",
  "upd-to",
  "zc",
  "zone-c",
  "ir",
  "inet-rtr",
  "an",
  "aut-num",
  "rt",
  "route",
  "ps",
  "peering-set",
  "is",
  "rtr-set",
  "la",
  "local-as",
  "localas",
  "mo",
  "member-of",
  "kc",
  "key-cert",
  "mh",
  "method",
  "ow",
  "owner",
  "at",
  "auth",
  "fp",
  "fingerpr",
  "ce",
  "certif",
  "in",
  "inetnum",
  "ml",
  "mnt-lower",
  "mu",
  "mnt-routes",
  "rb",
  "referral-by",
  "ak",
  "as-block",
  "ud",
  "delete",
  "uo",
  "override",
  NULL
}; /* Attribute_aliases */
