/* generated by 'Defs.java' - DO NOT HAND-EDIT */
#ifndef ATTRIBUTE_TAB_H
#define ATTRIBUTE_TAB_H

attribute_t attribute_tab[] = {
    {
        /* name */
        "address",
        /* id */
        0,
        /* altname */
        NULL,
        /* xmlname */
        "address",
        /* code */
        "ad",
        /* syntax_offset */
        9,   /* free-form */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "admin-c",
        /* id */
        1,
        /* altname */
        NULL,
        /* xmlname */
        "admin-c",
        /* code */
        "ac",
        /* syntax_offset */
        10,   /* nic-handle */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        "nh"
    },
    {
        /* name */
        "aggr-mtd",
        /* id */
        2,
        /* altname */
        NULL,
        /* xmlname */
        "aggr-mtd",
        /* code */
        "ag",
        /* syntax_offset */
        39,   /* aggr-mtd */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "aggr-bndry",
        /* id */
        3,
        /* altname */
        NULL,
        /* xmlname */
        "aggr-bndry",
        /* code */
        "ab",
        /* syntax_offset */
        40,   /* aggr-bndry */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "alias",
        /* id */
        4,
        /* altname */
        NULL,
        /* xmlname */
        "alias",
        /* code */
        "az",
        /* syntax_offset */
        17,   /* domain-name */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "as-name",
        /* id */
        5,
        /* altname */
        NULL,
        /* xmlname */
        "as-name",
        /* code */
        "aa",
        /* syntax_offset */
        11,   /* object-name */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "as-set",
        /* id */
        6,
        /* altname */
        NULL,
        /* xmlname */
        "as-set",
        /* code */
        "as",
        /* syntax_offset */
        1,   /* as-set */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "auth",
        /* id */
        7,
        /* altname */
        NULL,
        /* xmlname */
        "auth",
        /* code */
        "at",
        /* syntax_offset */
        23,   /* auth-scheme */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "author",
        /* id */
        8,
        /* altname */
        NULL,
        /* xmlname */
        "author",
        /* code */
        "ah",
        /* syntax_offset */
        10,   /* nic-handle */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        "nh"
    },
    {
        /* name */
        "certif",
        /* id */
        9,
        /* altname */
        NULL,
        /* xmlname */
        "certif",
        /* code */
        "ce",
        /* syntax_offset */
        30,   /* public-key */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "changed",
        /* id */
        10,
        /* altname */
        NULL,
        /* xmlname */
        "changed",
        /* code */
        "ch",
        /* syntax_offset */
        14,   /* changed */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "components",
        /* id */
        11,
        /* altname */
        NULL,
        /* xmlname */
        "components",
        /* code */
        "co",
        /* syntax_offset */
        38,   /* components */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "country",
        /* id */
        12,
        /* altname */
        NULL,
        /* xmlname */
        "country",
        /* code */
        "cy",
        /* syntax_offset */
        26,   /* country-code */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "cross-nfy",
        /* id */
        13,
        /* altname */
        NULL,
        /* xmlname */
        "cross-nfy",
        /* code */
        "cn",
        /* syntax_offset */
        10,   /* nic-handle */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        "nh"
    },
    {
        /* name */
        "default",
        /* id */
        14,
        /* altname */
        NULL,
        /* xmlname */
        "default",
        /* code */
        "df",
        /* syntax_offset */
        41,   /* default */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "descr",
        /* id */
        15,
        /* altname */
        NULL,
        /* xmlname */
        "descr",
        /* code */
        "de",
        /* syntax_offset */
        9,   /* free-form */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "dom-net",
        /* id */
        16,
        /* altname */
        NULL,
        /* xmlname */
        "dom-net",
        /* code */
        "di",
        /* syntax_offset */
        19,   /* ipv4-address */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        1,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "domain",
        /* id */
        17,
        /* altname */
        NULL,
        /* xmlname */
        "domain",
        /* code */
        "dn",
        /* syntax_offset */
        17,   /* domain-name */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "e-mail",
        /* id */
        18,
        /* altname */
        NULL,
        /* xmlname */
        "e-mail",
        /* code */
        "em",
        /* syntax_offset */
        13,   /* e-mail */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "export",
        /* id */
        19,
        /* altname */
        NULL,
        /* xmlname */
        "export",
        /* code */
        "ex",
        /* syntax_offset */
        42,   /* export */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "export-comps",
        /* id */
        20,
        /* altname */
        NULL,
        /* xmlname */
        "export-comps",
        /* code */
        "ec",
        /* syntax_offset */
        15,   /* filter */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "fax-no",
        /* id */
        21,
        /* altname */
        NULL,
        /* xmlname */
        "fax-no",
        /* code */
        "fx",
        /* syntax_offset */
        22,   /* telephone-number */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "filter",
        /* id */
        22,
        /* altname */
        NULL,
        /* xmlname */
        "filter",
        /* code */
        "fi",
        /* syntax_offset */
        15,   /* filter */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "filter-set",
        /* id */
        23,
        /* altname */
        NULL,
        /* xmlname */
        "filter-set",
        /* code */
        "fs",
        /* syntax_offset */
        0,   /* filter-set */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "fingerpr",
        /* id */
        24,
        /* altname */
        NULL,
        /* xmlname */
        "fingerpr",
        /* code */
        "fp",
        /* syntax_offset */
        31,   /* fingerpr */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "holes",
        /* id */
        25,
        /* altname */
        NULL,
        /* xmlname */
        "holes",
        /* code */
        "ho",
        /* syntax_offset */
        34,   /* address-prefix */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        1,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "ifaddr",
        /* id */
        26,
        /* altname */
        NULL,
        /* xmlname */
        "ifaddr",
        /* code */
        "if",
        /* syntax_offset */
        44,   /* ifaddr */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "import",
        /* id */
        27,
        /* altname */
        NULL,
        /* xmlname */
        "import",
        /* code */
        "ip",
        /* syntax_offset */
        43,   /* import */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "inetnum",
        /* id */
        28,
        /* altname */
        NULL,
        /* xmlname */
        "inetnum",
        /* code */
        "in",
        /* syntax_offset */
        24,   /* inetnum */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "inet6num",
        /* id */
        29,
        /* altname */
        NULL,
        /* xmlname */
        "inet6num",
        /* code */
        "i6",
        /* syntax_offset */
        25,   /* inet6num */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "inject",
        /* id */
        30,
        /* altname */
        NULL,
        /* xmlname */
        "inject",
        /* code */
        "ij",
        /* syntax_offset */
        45,   /* inject */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "key-cert",
        /* id */
        31,
        /* altname */
        NULL,
        /* xmlname */
        "key-cert",
        /* code */
        "kc",
        /* syntax_offset */
        32,   /* key-cert */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "mbrs-by-ref",
        /* id */
        32,
        /* altname */
        NULL,
        /* xmlname */
        "mbrs-by-ref",
        /* code */
        "mr",
        /* syntax_offset */
        8,   /* mbrs-by-ref */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        1,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "members",
        /* id */
        33,
        /* altname */
        NULL,
        /* xmlname */
        "members-as",
        /* code */
        "ms",
        /* syntax_offset */
        5,   /* members-as */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        1,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "members",
        /* id */
        33,
        /* altname */
        NULL,
        /* xmlname */
        "members-is",
        /* code */
        "ms",
        /* syntax_offset */
        6,   /* members-is */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        1,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "members",
        /* id */
        33,
        /* altname */
        NULL,
        /* xmlname */
        "members-rs",
        /* code */
        "ms",
        /* syntax_offset */
        7,   /* members-rs */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        1,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "method",
        /* id */
        34,
        /* altname */
        NULL,
        /* xmlname */
        "method",
        /* code */
        "mh",
        /* syntax_offset */
        33,   /* method */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "mnt-by",
        /* id */
        35,
        /* altname */
        NULL,
        /* xmlname */
        "mnt-by",
        /* code */
        "mb",
        /* syntax_offset */
        11,   /* object-name */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        1,
        /* is_ripe_list */
        0,
        /* foreign_code */
        "mt"
    },
    {
        /* name */
        "mnt-lower",
        /* id */
        36,
        /* altname */
        NULL,
        /* xmlname */
        "mnt-lower",
        /* code */
        "ml",
        /* syntax_offset */
        11,   /* object-name */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        1,
        /* is_ripe_list */
        0,
        /* foreign_code */
        "mt"
    },
    {
        /* name */
        "mnt-nfy",
        /* id */
        37,
        /* altname */
        NULL,
        /* xmlname */
        "mnt-nfy",
        /* code */
        "mn",
        /* syntax_offset */
        13,   /* e-mail */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "mnt-routes",
        /* id */
        38,
        /* altname */
        NULL,
        /* xmlname */
        "mnt-routes",
        /* code */
        "mu",
        /* syntax_offset */
        29,   /* mnt-routes */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        "mt"
    },
    {
        /* name */
        "mntner",
        /* id */
        39,
        /* altname */
        NULL,
        /* xmlname */
        "mntner",
        /* code */
        "mt",
        /* syntax_offset */
        11,   /* object-name */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "netname",
        /* id */
        40,
        /* altname */
        NULL,
        /* xmlname */
        "netname",
        /* code */
        "na",
        /* syntax_offset */
        12,   /* netname */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "nic-hdl",
        /* id */
        41,
        /* altname */
        NULL,
        /* xmlname */
        "nic-hdl",
        /* code */
        "nh",
        /* syntax_offset */
        10,   /* nic-handle */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "notify",
        /* id */
        42,
        /* altname */
        NULL,
        /* xmlname */
        "notify",
        /* code */
        "ny",
        /* syntax_offset */
        13,   /* e-mail */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "origin",
        /* id */
        43,
        /* altname */
        NULL,
        /* xmlname */
        "origin",
        /* code */
        "or",
        /* syntax_offset */
        35,   /* as-number */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "owner",
        /* id */
        44,
        /* altname */
        NULL,
        /* xmlname */
        "owner",
        /* code */
        "ow",
        /* syntax_offset */
        9,   /* free-form */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "peer",
        /* id */
        45,
        /* altname */
        NULL,
        /* xmlname */
        "peer",
        /* code */
        "pe",
        /* syntax_offset */
        47,   /* peer */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "peering",
        /* id */
        46,
        /* altname */
        NULL,
        /* xmlname */
        "peering",
        /* code */
        "pg",
        /* syntax_offset */
        46,   /* peering */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "person",
        /* id */
        47,
        /* altname */
        NULL,
        /* xmlname */
        "person",
        /* code */
        "pn",
        /* syntax_offset */
        21,   /* person-name */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "phone",
        /* id */
        48,
        /* altname */
        NULL,
        /* xmlname */
        "phone",
        /* code */
        "ph",
        /* syntax_offset */
        22,   /* telephone-number */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "referral-by",
        /* id */
        49,
        /* altname */
        NULL,
        /* xmlname */
        "referral-by",
        /* code */
        "rb",
        /* syntax_offset */
        11,   /* object-name */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        "mt"
    },
    {
        /* name */
        "remarks",
        /* id */
        50,
        /* altname */
        NULL,
        /* xmlname */
        "remarks",
        /* code */
        "rm",
        /* syntax_offset */
        9,   /* free-form */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "nserver",
        /* id */
        51,
        /* altname */
        NULL,
        /* xmlname */
        "nserver",
        /* code */
        "ns",
        /* syntax_offset */
        17,   /* domain-name */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        1,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "refer",
        /* id */
        52,
        /* altname */
        NULL,
        /* xmlname */
        "refer",
        /* code */
        "rf",
        /* syntax_offset */
        20,   /* refer */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "rev-srv",
        /* id */
        53,
        /* altname */
        NULL,
        /* xmlname */
        "rev-srv",
        /* code */
        "rz",
        /* syntax_offset */
        17,   /* domain-name */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        1,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "role",
        /* id */
        54,
        /* altname */
        NULL,
        /* xmlname */
        "role",
        /* code */
        "ro",
        /* syntax_offset */
        21,   /* person-name */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "route-set",
        /* id */
        55,
        /* altname */
        NULL,
        /* xmlname */
        "route-set",
        /* code */
        "rs",
        /* syntax_offset */
        2,   /* route-set */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "source",
        /* id */
        56,
        /* altname */
        NULL,
        /* xmlname */
        "source",
        /* code */
        "so",
        /* syntax_offset */
        16,   /* registry-name */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "status",
        /* id */
        57,
        /* altname */
        NULL,
        /* xmlname */
        "status-in",
        /* code */
        "st",
        /* syntax_offset */
        27,   /* status-in */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "status",
        /* id */
        57,
        /* altname */
        NULL,
        /* xmlname */
        "status-i6",
        /* code */
        "st",
        /* syntax_offset */
        28,   /* status-i6 */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "sub-dom",
        /* id */
        58,
        /* altname */
        NULL,
        /* xmlname */
        "sub-dom",
        /* code */
        "sd",
        /* syntax_offset */
        18,   /* subdomain-name */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        1,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "tech-c",
        /* id */
        59,
        /* altname */
        NULL,
        /* xmlname */
        "tech-c",
        /* code */
        "tc",
        /* syntax_offset */
        10,   /* nic-handle */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        "nh"
    },
    {
        /* name */
        "text",
        /* id */
        60,
        /* altname */
        NULL,
        /* xmlname */
        "text",
        /* code */
        "tx",
        /* syntax_offset */
        9,   /* free-form */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "trouble",
        /* id */
        61,
        /* altname */
        NULL,
        /* xmlname */
        "trouble",
        /* code */
        "tb",
        /* syntax_offset */
        9,   /* free-form */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "upd-to",
        /* id */
        62,
        /* altname */
        NULL,
        /* xmlname */
        "upd-to",
        /* code */
        "dt",
        /* syntax_offset */
        13,   /* e-mail */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "zone-c",
        /* id */
        63,
        /* altname */
        NULL,
        /* xmlname */
        "zone-c",
        /* code */
        "zc",
        /* syntax_offset */
        10,   /* nic-handle */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        "nh"
    },
    {
        /* name */
        "inet-rtr",
        /* id */
        64,
        /* altname */
        NULL,
        /* xmlname */
        "inet-rtr",
        /* code */
        "ir",
        /* syntax_offset */
        17,   /* domain-name */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "as-block",
        /* id */
        65,
        /* altname */
        NULL,
        /* xmlname */
        "as-block",
        /* code */
        "ak",
        /* syntax_offset */
        36,   /* as-block */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "aut-num",
        /* id */
        66,
        /* altname */
        NULL,
        /* xmlname */
        "aut-num",
        /* code */
        "an",
        /* syntax_offset */
        35,   /* as-number */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "limerick",
        /* id */
        67,
        /* altname */
        NULL,
        /* xmlname */
        "limerick",
        /* code */
        "li",
        /* syntax_offset */
        37,   /* limerick */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "route",
        /* id */
        68,
        /* altname */
        NULL,
        /* xmlname */
        "route",
        /* code */
        "rt",
        /* syntax_offset */
        34,   /* address-prefix */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "peering-set",
        /* id */
        69,
        /* altname */
        NULL,
        /* xmlname */
        "peering-set",
        /* code */
        "ps",
        /* syntax_offset */
        3,   /* peering-set */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "rtr-set",
        /* id */
        70,
        /* altname */
        NULL,
        /* xmlname */
        "rtr-set",
        /* code */
        "is",
        /* syntax_offset */
        4,   /* rtr-set */
        /* is_lookup */
        1,
        /* is_inverse */
        0,
        /* is_primary */
        1,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "local-as",
        /* id */
        71,
        /* altname */
        "localas",
        /* xmlname */
        "local-as",
        /* code */
        "la",
        /* syntax_offset */
        35,   /* as-number */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "member-of",
        /* id */
        72,
        /* altname */
        NULL,
        /* xmlname */
        "member-of-ir",
        /* code */
        "mo",
        /* syntax_offset */
        4,   /* rtr-set */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        1,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "member-of",
        /* id */
        72,
        /* altname */
        NULL,
        /* xmlname */
        "member-of-an",
        /* code */
        "mo",
        /* syntax_offset */
        1,   /* as-set */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        1,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "member-of",
        /* id */
        72,
        /* altname */
        NULL,
        /* xmlname */
        "member-of-rt",
        /* code */
        "mo",
        /* syntax_offset */
        2,   /* route-set */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        1,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "cross-mnt",
        /* id */
        73,
        /* altname */
        NULL,
        /* xmlname */
        "cross-mnt",
        /* code */
        "ct",
        /* syntax_offset */
        11,   /* object-name */
        /* is_lookup */
        0,
        /* is_inverse */
        1,
        /* is_primary */
        0,
        /* is_list */
        1,
        /* is_ripe_list */
        0,
        /* foreign_code */
        "mt"
    },
    {
        /* name */
        "delete",
        /* id */
        74,
        /* altname */
        NULL,
        /* xmlname */
        "delete",
        /* code */
        "ud",
        /* syntax_offset */
        9,   /* free-form */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    },
    {
        /* name */
        "override",
        /* id */
        75,
        /* altname */
        NULL,
        /* xmlname */
        "override",
        /* code */
        "uo",
        /* syntax_offset */
        9,   /* free-form */
        /* is_lookup */
        0,
        /* is_inverse */
        0,
        /* is_primary */
        0,
        /* is_list */
        0,
        /* is_ripe_list */
        0,
        /* foreign_code */
        NULL,
    }
};

#endif /* ATTRIBUTE_TAB_H */
