/* generated by 'Defs.java' - DO NOT HAND-EDIT */
#ifndef CLASS_TAB_H
#define CLASS_TAB_H

class_t class_tab[] = {
    {
        /* name */
        "as-block",
        /* id */
        0,
        /* num_attr */
        13,
        /* attr[] */
        {
            /* attribute: as-block (as-block) */
            {
                /* offset */
                68,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: country (country) */
            {
                /* offset */
                12,
                /* choice */
                ATTR_GENERATED,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: tech-c (tech-c) */
            {
                /* offset */
                62,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-lower (mnt-lower) */
            {
                /* offset */
                38,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "as-set",
        /* id */
        1,
        /* num_attr */
        13,
        /* attr[] */
        {
            /* attribute: as-set (as-set) */
            {
                /* offset */
                6,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: members (members-as) */
            {
                /* offset */
                33,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mbrs-by-ref (mbrs-by-ref) */
            {
                /* offset */
                32,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: tech-c (tech-c) */
            {
                /* offset */
                62,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "aut-num",
        /* id */
        2,
        /* num_attr */
        21,
        /* attr[] */
        {
            /* attribute: aut-num (aut-num) */
            {
                /* offset */
                69,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: as-name (as-name) */
            {
                /* offset */
                5,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: country (country) */
            {
                /* offset */
                12,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: member-of (member-of-an) */
            {
                /* offset */
                76,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: import (import) */
            {
                /* offset */
                27,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: export (export) */
            {
                /* offset */
                19,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: default (default) */
            {
                /* offset */
                14,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: tech-c (tech-c) */
            {
                /* offset */
                62,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: cross-mnt (cross-mnt) */
            {
                /* offset */
                78,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: cross-nfy (cross-nfy) */
            {
                /* offset */
                13,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-lower (mnt-lower) */
            {
                /* offset */
                38,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-routes (mnt-routes) */
            {
                /* offset */
                40,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "domain",
        /* id */
        3,
        /* num_attr */
        17,
        /* attr[] */
        {
            /* attribute: domain (domain) */
            {
                /* offset */
                17,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: tech-c (tech-c) */
            {
                /* offset */
                62,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: zone-c (zone-c) */
            {
                /* offset */
                66,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: nserver (nserver) */
            {
                /* offset */
                53,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: sub-dom (sub-dom) */
            {
                /* offset */
                61,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: dom-net (dom-net) */
            {
                /* offset */
                16,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-lower (mnt-lower) */
            {
                /* offset */
                38,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: refer (refer) */
            {
                /* offset */
                54,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "inet-rtr",
        /* id */
        4,
        /* num_attr */
        16,
        /* attr[] */
        {
            /* attribute: inet-rtr (inet-rtr) */
            {
                /* offset */
                67,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: alias (alias) */
            {
                /* offset */
                4,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: local-as (local-as) */
            {
                /* offset */
                74,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: ifaddr (ifaddr) */
            {
                /* offset */
                26,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: peer (peer) */
            {
                /* offset */
                47,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: member-of (member-of-ir) */
            {
                /* offset */
                75,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: tech-c (tech-c) */
            {
                /* offset */
                62,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "inet6num",
        /* id */
        5,
        /* num_attr */
        16,
        /* attr[] */
        {
            /* attribute: inet6num (inet6num) */
            {
                /* offset */
                29,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: netname (netname) */
            {
                /* offset */
                42,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: country (country) */
            {
                /* offset */
                12,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: tech-c (tech-c) */
            {
                /* offset */
                62,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: rev-srv (rev-srv) */
            {
                /* offset */
                55,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: status (status-i6) */
            {
                /* offset */
                60,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-lower (mnt-lower) */
            {
                /* offset */
                38,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "inetnum",
        /* id */
        6,
        /* num_attr */
        17,
        /* attr[] */
        {
            /* attribute: inetnum (inetnum) */
            {
                /* offset */
                28,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: netname (netname) */
            {
                /* offset */
                42,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: country (country) */
            {
                /* offset */
                12,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: tech-c (tech-c) */
            {
                /* offset */
                62,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: rev-srv (rev-srv) */
            {
                /* offset */
                55,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: status (status-in) */
            {
                /* offset */
                59,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-lower (mnt-lower) */
            {
                /* offset */
                38,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-routes (mnt-routes) */
            {
                /* offset */
                40,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "key-cert",
        /* id */
        7,
        /* num_attr */
        12,
        /* attr[] */
        {
            /* attribute: key-cert (key-cert) */
            {
                /* offset */
                31,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: method (method) */
            {
                /* offset */
                36,
                /* choice */
                ATTR_GENERATED,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: owner (owner) */
            {
                /* offset */
                46,
                /* choice */
                ATTR_GENERATED,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: fingerpr (fingerpr) */
            {
                /* offset */
                24,
                /* choice */
                ATTR_GENERATED,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: certif (certif) */
            {
                /* offset */
                9,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "limerick",
        /* id */
        8,
        /* num_attr */
        12,
        /* attr[] */
        {
            /* attribute: limerick (limerick) */
            {
                /* offset */
                70,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: text (text) */
            {
                /* offset */
                63,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: author (author) */
            {
                /* offset */
                8,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "mntner",
        /* id */
        9,
        /* num_attr */
        16,
        /* attr[] */
        {
            /* attribute: mntner (mntner) */
            {
                /* offset */
                41,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: country (country) */
            {
                /* offset */
                12,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: tech-c (tech-c) */
            {
                /* offset */
                62,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: upd-to (upd-to) */
            {
                /* offset */
                65,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-nfy (mnt-nfy) */
            {
                /* offset */
                39,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: auth (auth) */
            {
                /* offset */
                7,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: referral-by (referral-by) */
            {
                /* offset */
                51,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "person",
        /* id */
        10,
        /* num_attr */
        14,
        /* attr[] */
        {
            /* attribute: person (person) */
            {
                /* offset */
                49,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: address (address) */
            {
                /* offset */
                0,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: country (country) */
            {
                /* offset */
                12,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: phone (phone) */
            {
                /* offset */
                50,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: fax-no (fax-no) */
            {
                /* offset */
                21,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: e-mail (e-mail) */
            {
                /* offset */
                18,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: nic-hdl (nic-hdl) */
            {
                /* offset */
                43,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "role",
        /* id */
        11,
        /* num_attr */
        17,
        /* attr[] */
        {
            /* attribute: role (role) */
            {
                /* offset */
                56,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: address (address) */
            {
                /* offset */
                0,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: country (country) */
            {
                /* offset */
                12,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: phone (phone) */
            {
                /* offset */
                50,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: fax-no (fax-no) */
            {
                /* offset */
                21,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: e-mail (e-mail) */
            {
                /* offset */
                18,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: trouble (trouble) */
            {
                /* offset */
                64,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: tech-c (tech-c) */
            {
                /* offset */
                62,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: nic-hdl (nic-hdl) */
            {
                /* offset */
                43,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "route",
        /* id */
        12,
        /* num_attr */
        22,
        /* attr[] */
        {
            /* attribute: route (route) */
            {
                /* offset */
                71,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: country (country) */
            {
                /* offset */
                12,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: origin (origin) */
            {
                /* offset */
                45,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: holes (holes) */
            {
                /* offset */
                25,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: member-of (member-of-rt) */
            {
                /* offset */
                77,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: inject (inject) */
            {
                /* offset */
                30,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: aggr-mtd (aggr-mtd) */
            {
                /* offset */
                2,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: aggr-bndry (aggr-bndry) */
            {
                /* offset */
                3,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: export-comps (export-comps) */
            {
                /* offset */
                20,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: components (components) */
            {
                /* offset */
                11,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: cross-mnt (cross-mnt) */
            {
                /* offset */
                78,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: cross-nfy (cross-nfy) */
            {
                /* offset */
                13,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-lower (mnt-lower) */
            {
                /* offset */
                38,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-routes (mnt-routes) */
            {
                /* offset */
                40,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "route-set",
        /* id */
        13,
        /* num_attr */
        13,
        /* attr[] */
        {
            /* attribute: route-set (route-set) */
            {
                /* offset */
                57,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: members (members-rs) */
            {
                /* offset */
                35,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mbrs-by-ref (mbrs-by-ref) */
            {
                /* offset */
                32,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: tech-c (tech-c) */
            {
                /* offset */
                62,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "filter-set",
        /* id */
        14,
        /* num_attr */
        12,
        /* attr[] */
        {
            /* attribute: filter-set (filter-set) */
            {
                /* offset */
                23,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: filter (filter) */
            {
                /* offset */
                22,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: tech-c (tech-c) */
            {
                /* offset */
                62,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "peering-set",
        /* id */
        15,
        /* num_attr */
        12,
        /* attr[] */
        {
            /* attribute: peering-set (peering-set) */
            {
                /* offset */
                72,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: peering (peering) */
            {
                /* offset */
                48,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: tech-c (tech-c) */
            {
                /* offset */
                62,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    },
    {
        /* name */
        "rtr-set",
        /* id */
        16,
        /* num_attr */
        13,
        /* attr[] */
        {
            /* attribute: rtr-set (rtr-set) */
            {
                /* offset */
                73,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: descr (descr) */
            {
                /* offset */
                15,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: members (members-is) */
            {
                /* offset */
                34,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mbrs-by-ref (mbrs-by-ref) */
            {
                /* offset */
                32,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: remarks (remarks) */
            {
                /* offset */
                52,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: tech-c (tech-c) */
            {
                /* offset */
                62,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: admin-c (admin-c) */
            {
                /* offset */
                1,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: notify (notify) */
            {
                /* offset */
                44,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: mnt-by (mnt-by) */
            {
                /* offset */
                37,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: changed (changed) */
            {
                /* offset */
                10,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_MULTIPLE
            },
            /* attribute: source (source) */
            {
                /* offset */
                58,
                /* choice */
                ATTR_MANDATORY,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: delete (delete) */
            {
                /* offset */
                79,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            },
            /* attribute: override (override) */
            {
                /* offset */
                80,
                /* choice */
                ATTR_OPTIONAL,
                /* number */
                ATTR_SINGLE
            }
        },
        /* attr_hash (set by class_init()) */
        NULL
    }
};

#endif /* CLASS_TAB_H */
