DF_Load_t DF_radix_load[] = 
{
  { A_RT, RX_FAM_RT,
	"SELECT  object_id,prefix,prefix_length,origin                 FROM    route                 WHERE   thread_id = 0",
	NULL
  },
  { A_IN, RX_FAM_IN,
	"SELECT  object_id,begin_in,end_in                 FROM    inetnum                 WHERE   thread_id = 0",
	NULL
  },
  { -1, -1, NULL, NULL }
};
