
/*  A Bison parser, made from er.yacc
    by GNU Bison version 1.28  */

#define YYBISON 1  /* Identify Bison output.  */

#define yyparse er_yyparse
#define yylex er_yylex
#define yyerror er_yyerror
#define yylval er_yylval
#define yychar er_yychar
#define yydebug er_yydebug
#define yynerrs er_yynerrs
#define	EOL	257
#define	K_SOCK	258
#define	K_NAME	259
#define	K_FAC	260
#define	K_ADD	261
#define	K_RM	262
#define	K_CREATE	263
#define	K_DELETE	264
#define	K_MODIFY	265
#define	K_AUTO	266
#define	K_ASP	267
#define	K_SEV	268
#define	K_THR	269
#define	K_FORMAT	270
#define	K_SELF	271
#define	K_DATE	272
#define	K_ALL	273
#define	K_EXEC	274
#define	K_PATH	275
#define	STRING	276
#define	NUMBER	277
#define	HEXNUM	278



/* NOTE: We'd like to use "rip.h", but "mm.h" defines STRING which 
         collides with the STRING here */
#include "erroutines.h"
#include "memwrap.h"
#include "bitmask.h"
#include "er_yacc_helper.h"
#include "er_arrays.h"
#include "er_macro.h"
#include "er_paths.h"

#include <stdlib.h> 
#include <string.h>

#define YYPARSE_PARAM ypar
#define YYLEX_PARAM   ypar
#define YYDEBUG 0
/* #define YYERROR_VERBOSE  */

#define er_yyerror(a) myerror(ypar,a)

#define adderr(a,b) {strcat(a->errtxt,b);}

#define myfree(mem) {lexerr_t *erst = ypar; free(mem); erst->token = NULL} 

#define MYABORT { mycleanup(ypar); YYABORT; }


typedef union {
      int inum;
      char *text;
} YYSTYPE;
#include <stdio.h>

#ifndef __cplusplus
#ifndef __STDC__
#define const
#endif
#endif



#define	YYFINAL		85
#define	YYFLAG		-32768
#define	YYNTBASE	31

#define YYTRANSLATE(x) ((unsigned)(x) <= 278 ? yytranslate[x] : 62)

static const char yytranslate[] = {     0,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,    28,
    29,     2,     2,     2,    30,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,    25,    27,    26,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     1,     3,     4,     5,     6,
     7,     8,     9,    10,    11,    12,    13,    14,    15,    16,
    17,    18,    19,    20,    21,    22,    23,    24
};

#if YYDEBUG != 0
static const short yyprhs[] = {     0,
     0,     2,     5,     7,     8,    14,    17,    21,    26,    31,
    33,    35,    40,    42,    44,    46,    48,    51,    53,    55,
    59,    61,    64,    68,    71,    72,    74,    78,    79,    81,
    83,    86,    88,    90,    93,    97,   102,   104,   107,   110,
   111,   114,   115,   118,   120,   124,   126,   128,   132,   134,
   135,   137,   138,   140,   142,   144,   148,   150
};

static const short yyrhs[] = {    32,
     0,    32,     3,     0,     1,     0,     0,     9,    34,    33,
    35,    47,     0,    10,    34,     0,    11,    34,    35,     0,
    11,    34,     8,    46,     0,    11,    34,     7,    47,     0,
    22,     0,    12,     0,    25,    37,    36,    26,     0,    40,
     0,    41,     0,    43,     0,     1,     0,    16,    38,     0,
     1,     0,    39,     0,    38,    27,    39,     0,    22,     0,
     4,    23,     0,     5,    22,    42,     0,     5,     1,     0,
     0,    18,     0,    20,    44,    45,     0,     0,    21,     0,
    22,     0,    45,    22,     0,    23,     0,    48,     0,    47,
    48,     0,    28,    49,    29,     0,    50,    52,    51,    53,
     0,     1,     0,     6,    54,     0,    14,    56,     0,     0,
    13,    60,     0,     0,    15,    17,     0,    55,     0,    54,
    27,    55,     0,    19,     0,    22,     0,    57,    30,    58,
     0,    59,     0,     0,    22,     0,     0,    22,     0,    22,
     0,    61,     0,    60,    27,    61,     0,    22,     0,    24,
     0
};

#endif

#if YYDEBUG != 0
static const short yyrline[] = { 0,
    80,    81,    82,    87,    92,   121,   134,   149,   165,   181,
   182,   187,   192,   193,   194,   195,   200,   201,   206,   207,
   210,   227,   235,   242,   247,   247,   253,   259,   259,   265,
   270,   278,   281,   282,   285,   294,   295,   302,   304,   306,
   307,   309,   310,   316,   317,   322,   326,   343,   344,   347,
   350,   356,   359,   365,   378,   379,   384,   399
};
#endif


#if YYDEBUG != 0 || defined (YYERROR_VERBOSE)

static const char * const yytname[] = {   "$","error","$undefined.","EOL","K_SOCK",
"K_NAME","K_FAC","K_ADD","K_RM","K_CREATE","K_DELETE","K_MODIFY","K_AUTO","K_ASP",
"K_SEV","K_THR","K_FORMAT","K_SELF","K_DATE","K_ALL","K_EXEC","K_PATH","STRING",
"NUMBER","HEXNUM","'{'","'}'","'|'","'('","')'","'-'","fullline","stmt","@1",
"pathid","path","pathspec","formatspec","formlist","oneform","sockspec","namespec",
"nameoptions","execspec","opt_path","execwords","filterid","filters","filter",
"filtspec","mand_fac","mand_sev","opt_asp","opt_thr","facspec","onefac","sevspec",
"sevmin","sevmax","seveql","aspspec","oneasp", NULL
};
#endif

static const short yyr1[] = {     0,
    31,    31,    31,    33,    32,    32,    32,    32,    32,    34,
    34,    35,    36,    36,    36,    36,    37,    37,    38,    38,
    39,    40,    41,    41,    42,    42,    43,    44,    44,    45,
    45,    46,    47,    47,    48,    49,    49,    50,    51,    52,
    52,    53,    53,    54,    54,    55,    55,    56,    56,    57,
    57,    58,    58,    59,    60,    60,    61,    61
};

static const short yyr2[] = {     0,
     1,     2,     1,     0,     5,     2,     3,     4,     4,     1,
     1,     4,     1,     1,     1,     1,     2,     1,     1,     3,
     1,     2,     3,     2,     0,     1,     3,     0,     1,     1,
     2,     1,     1,     2,     3,     4,     1,     2,     2,     0,
     2,     0,     2,     1,     3,     1,     1,     3,     1,     0,
     1,     0,     1,     1,     1,     3,     1,     1
};

static const short yydefact[] = {     0,
     3,     0,     0,     0,     1,    11,    10,     4,     6,     0,
     2,     0,     0,     0,     0,     7,     0,     0,     9,    33,
    32,     8,    18,     0,     0,     5,    37,     0,     0,    40,
    34,    21,    17,    19,    16,     0,     0,    28,     0,    13,
    14,    15,    46,    47,    38,    44,    35,     0,     0,     0,
    22,    24,    25,    29,     0,    12,     0,    57,    58,    41,
    55,    50,    42,    20,    26,    23,    30,    27,    45,     0,
    54,    39,     0,    49,     0,    36,    31,    56,    52,    43,
    53,    48,     0,     0,     0
};

static const short yydefgoto[] = {    83,
     5,    12,     8,    16,    39,    25,    33,    34,    40,    41,
    66,    42,    55,    68,    22,    19,    20,    29,    30,    63,
    49,    76,    45,    46,    72,    73,    82,    74,    60,    61
};

static const short yypact[] = {     5,
-32768,   -10,   -10,   -10,     4,-32768,-32768,-32768,-32768,     1,
-32768,    -2,     6,     7,     2,-32768,     6,    18,     6,-32768,
-32768,-32768,-32768,     9,     0,     6,-32768,     3,     8,    16,
-32768,-32768,    11,-32768,-32768,    10,    -1,    14,    13,-32768,
-32768,-32768,-32768,-32768,    15,-32768,-32768,   -11,    22,     9,
-32768,-32768,    23,-32768,    21,-32768,     3,-32768,-32768,    17,
-32768,    25,    30,-32768,-32768,-32768,-32768,    26,-32768,   -11,
    19,-32768,    20,-32768,    29,-32768,-32768,-32768,    31,-32768,
-32768,-32768,    32,    40,-32768
};

static const short yypgoto[] = {-32768,
-32768,-32768,    24,    39,-32768,-32768,-32768,    12,-32768,-32768,
-32768,-32768,-32768,-32768,-32768,    35,    -9,-32768,-32768,-32768,
-32768,-32768,-32768,    -3,-32768,-32768,-32768,-32768,-32768,   -15
};


#define	YYLAST		62


static const short yytable[] = {    52,
    35,     6,    23,    36,    37,     1,    11,    13,    14,    31,
    58,     7,    59,     2,     3,     4,    31,    24,    27,    38,
    53,    43,    15,    28,    44,    15,     9,    10,    48,    21,
    32,    84,    51,    18,    54,    62,    47,    50,    56,    85,
    65,    57,    67,    70,    75,    80,    71,    77,   -51,    79,
    17,    26,    81,    69,    78,     0,     0,     0,     0,     0,
     0,    64
};

static const short yycheck[] = {     1,
     1,    12,     1,     4,     5,     1,     3,     7,     8,    19,
    22,    22,    24,     9,    10,    11,    26,    16,     1,    20,
    22,    19,    25,     6,    22,    25,     3,     4,    13,    23,
    22,     0,    23,    28,    21,    14,    29,    27,    26,     0,
    18,    27,    22,    27,    15,    17,    22,    22,    30,    30,
    12,    17,    22,    57,    70,    -1,    -1,    -1,    -1,    -1,
    -1,    50
};
#define YYPURE 1

/* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */

/* This file comes from bison-1.28.  */

/* Skeleton output parser for bison,
   Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

/* As a special exception, when this file is copied by Bison into a
   Bison output file, you may use that output file without restriction.
   This special exception was added by the Free Software Foundation
   in version 1.24 of Bison.  */

/* This is the parser code that is written into each bison parser
  when the %semantic_parser declaration is not specified in the grammar.
  It was written by Richard Stallman by simplifying the hairy parser
  used when %semantic_parser is specified.  */

#ifndef YYSTACK_USE_ALLOCA
#ifdef alloca
#define YYSTACK_USE_ALLOCA
#else /* alloca not defined */
#ifdef __GNUC__
#define YYSTACK_USE_ALLOCA
#define alloca __builtin_alloca
#else /* not GNU C.  */
#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi) || (defined (__sun) && defined (__i386))
#define YYSTACK_USE_ALLOCA
#include <alloca.h>
#else /* not sparc */
/* We think this test detects Watcom and Microsoft C.  */
/* This used to test MSDOS, but that is a bad idea
   since that symbol is in the user namespace.  */
#if (defined (_MSDOS) || defined (_MSDOS_)) && !defined (__TURBOC__)
#if 0 /* No need for malloc.h, which pollutes the namespace;
	 instead, just don't use alloca.  */
#include <malloc.h>
#endif
#else /* not MSDOS, or __TURBOC__ */
#if defined(_AIX)
/* I don't know what this was needed for, but it pollutes the namespace.
   So I turned it off.   rms, 2 May 1997.  */
/* #include <malloc.h>  */
 #pragma alloca
#define YYSTACK_USE_ALLOCA
#else /* not MSDOS, or __TURBOC__, or _AIX */
#if 0
#ifdef __hpux /* haible@ilog.fr says this works for HPUX 9.05 and up,
		 and on HPUX 10.  Eventually we can turn this on.  */
#define YYSTACK_USE_ALLOCA
#define alloca __builtin_alloca
#endif /* __hpux */
#endif
#endif /* not _AIX */
#endif /* not MSDOS, or __TURBOC__ */
#endif /* not sparc */
#endif /* not GNU C */
#endif /* alloca not defined */
#endif /* YYSTACK_USE_ALLOCA not defined */

#ifdef YYSTACK_USE_ALLOCA
#define YYSTACK_ALLOC alloca
#else
#define YYSTACK_ALLOC malloc
#endif

/* Note: there must be only one dollar sign in this file.
   It is replaced by the list of actions, each action
   as one case of the switch.  */

#define yyerrok		(yyerrstatus = 0)
#define yyclearin	(yychar = YYEMPTY)
#define YYEMPTY		-2
#define YYEOF		0
#define YYACCEPT	goto yyacceptlab
#define YYABORT 	goto yyabortlab
#define YYERROR		goto yyerrlab1
/* Like YYERROR except do call yyerror.
   This remains here temporarily to ease the
   transition to the new meaning of YYERROR, for GCC.
   Once GCC version 2 has supplanted version 1, this can go.  */
#define YYFAIL		goto yyerrlab
#define YYRECOVERING()  (!!yyerrstatus)
#define YYBACKUP(token, value) \
do								\
  if (yychar == YYEMPTY && yylen == 1)				\
    { yychar = (token), yylval = (value);			\
      yychar1 = YYTRANSLATE (yychar);				\
      YYPOPSTACK;						\
      goto yybackup;						\
    }								\
  else								\
    { yyerror ("syntax error: cannot back up"); YYERROR; }	\
while (0)

#define YYTERROR	1
#define YYERRCODE	256

#ifndef YYPURE
#define YYLEX		yylex()
#endif

#ifdef YYPURE
#ifdef YYLSP_NEEDED
#ifdef YYLEX_PARAM
#define YYLEX		yylex(&yylval, &yylloc, YYLEX_PARAM)
#else
#define YYLEX		yylex(&yylval, &yylloc)
#endif
#else /* not YYLSP_NEEDED */
#ifdef YYLEX_PARAM
#define YYLEX		yylex(&yylval, YYLEX_PARAM)
#else
#define YYLEX		yylex(&yylval)
#endif
#endif /* not YYLSP_NEEDED */
#endif

/* If nonreentrant, generate the variables here */

#ifndef YYPURE

int	yychar;			/*  the lookahead symbol		*/
YYSTYPE	yylval;			/*  the semantic value of the		*/
				/*  lookahead symbol			*/

#ifdef YYLSP_NEEDED
YYLTYPE yylloc;			/*  location data for the lookahead	*/
				/*  symbol				*/
#endif

int yynerrs;			/*  number of parse errors so far       */
#endif  /* not YYPURE */

#if YYDEBUG != 0
int yydebug;			/*  nonzero means print parse trace	*/
/* Since this is uninitialized, it does not stop multiple parsers
   from coexisting.  */
#endif

/*  YYINITDEPTH indicates the initial size of the parser's stacks	*/

#ifndef	YYINITDEPTH
#define YYINITDEPTH 200
#endif

/*  YYMAXDEPTH is the maximum size the stacks can grow to
    (effective only if the built-in stack extension method is used).  */

#if YYMAXDEPTH == 0
#undef YYMAXDEPTH
#endif

#ifndef YYMAXDEPTH
#define YYMAXDEPTH 10000
#endif

/* Define __yy_memcpy.  Note that the size argument
   should be passed with type unsigned int, because that is what the non-GCC
   definitions require.  With GCC, __builtin_memcpy takes an arg
   of type size_t, but it can handle unsigned int.  */

#if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
#define __yy_memcpy(TO,FROM,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
#else				/* not GNU C or C++ */
#ifndef __cplusplus

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_memcpy (to, from, count)
     char *to;
     char *from;
     unsigned int count;
{
  register char *f = from;
  register char *t = to;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#else /* __cplusplus */

/* This is the most reliable way to avoid incompatibilities
   in available built-in functions on various systems.  */
static void
__yy_memcpy (char *to, char *from, unsigned int count)
{
  register char *t = to;
  register char *f = from;
  register int i = count;

  while (i-- > 0)
    *t++ = *f++;
}

#endif
#endif



/* The user can define YYPARSE_PARAM as the name of an argument to be passed
   into yyparse.  The argument should have type void *.
   It should actually point to an object.
   Grammar actions can access the variable by casting it
   to the proper pointer type.  */

#ifdef YYPARSE_PARAM
#ifdef __cplusplus
#define YYPARSE_PARAM_ARG void *YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else /* not __cplusplus */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL void *YYPARSE_PARAM;
#endif /* not __cplusplus */
#else /* not YYPARSE_PARAM */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif /* not YYPARSE_PARAM */

/* Prevent warning if -Wstrict-prototypes.  */
#ifdef __GNUC__
#ifdef YYPARSE_PARAM
int yyparse (void *);
#else
int yyparse (void);
#endif
#endif

int
yyparse(YYPARSE_PARAM_ARG)
     YYPARSE_PARAM_DECL
{
  register int yystate;
  register int yyn;
  register short *yyssp;
  register YYSTYPE *yyvsp;
  int yyerrstatus;	/*  number of tokens to shift before error messages enabled */
  int yychar1 = 0;		/*  lookahead token as an internal (translated) token number */

  short	yyssa[YYINITDEPTH];	/*  the state stack			*/
  YYSTYPE yyvsa[YYINITDEPTH];	/*  the semantic value stack		*/

  short *yyss = yyssa;		/*  refer to the stacks thru separate pointers */
  YYSTYPE *yyvs = yyvsa;	/*  to allow yyoverflow to reallocate them elsewhere */

#ifdef YYLSP_NEEDED
  YYLTYPE yylsa[YYINITDEPTH];	/*  the location stack			*/
  YYLTYPE *yyls = yylsa;
  YYLTYPE *yylsp;

#define YYPOPSTACK   (yyvsp--, yyssp--, yylsp--)
#else
#define YYPOPSTACK   (yyvsp--, yyssp--)
#endif

  int yystacksize = YYINITDEPTH;
  int yyfree_stacks = 0;

#ifdef YYPURE
  int yychar;
  YYSTYPE yylval;
  int yynerrs;
#ifdef YYLSP_NEEDED
  YYLTYPE yylloc;
#endif
#endif

  YYSTYPE yyval;		/*  the variable used to return		*/
				/*  semantic values from the action	*/
				/*  routines				*/

  int yylen;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Starting parse\n");
#endif

  yystate = 0;
  yyerrstatus = 0;
  yynerrs = 0;
  yychar = YYEMPTY;		/* Cause a token to be read.  */

  /* Initialize stack pointers.
     Waste one element of value and location stack
     so that they stay on the same level as the state stack.
     The wasted elements are never initialized.  */

  yyssp = yyss - 1;
  yyvsp = yyvs;
#ifdef YYLSP_NEEDED
  yylsp = yyls;
#endif

/* Push a new state, which is found in  yystate  .  */
/* In all cases, when you get here, the value and location stacks
   have just been pushed. so pushing a state here evens the stacks.  */
yynewstate:

  *++yyssp = yystate;

  if (yyssp >= yyss + yystacksize - 1)
    {
      /* Give user a chance to reallocate the stack */
      /* Use copies of these so that the &'s don't force the real ones into memory. */
      YYSTYPE *yyvs1 = yyvs;
      short *yyss1 = yyss;
#ifdef YYLSP_NEEDED
      YYLTYPE *yyls1 = yyls;
#endif

      /* Get the current used size of the three stacks, in elements.  */
      int size = yyssp - yyss + 1;

#ifdef yyoverflow
      /* Each stack pointer address is followed by the size of
	 the data in use in that stack, in bytes.  */
#ifdef YYLSP_NEEDED
      /* This used to be a conditional around just the two extra args,
	 but that might be undefined if yyoverflow is a macro.  */
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yyls1, size * sizeof (*yylsp),
		 &yystacksize);
#else
      yyoverflow("parser stack overflow",
		 &yyss1, size * sizeof (*yyssp),
		 &yyvs1, size * sizeof (*yyvsp),
		 &yystacksize);
#endif

      yyss = yyss1; yyvs = yyvs1;
#ifdef YYLSP_NEEDED
      yyls = yyls1;
#endif
#else /* no yyoverflow */
      /* Extend the stack our own way.  */
      if (yystacksize >= YYMAXDEPTH)
	{
	  yyerror("parser stack overflow");
	  if (yyfree_stacks)
	    {
	      free (yyss);
	      free (yyvs);
#ifdef YYLSP_NEEDED
	      free (yyls);
#endif
	    }
	  return 2;
	}
      yystacksize *= 2;
      if (yystacksize > YYMAXDEPTH)
	yystacksize = YYMAXDEPTH;
#ifndef YYSTACK_USE_ALLOCA
      yyfree_stacks = 1;
#endif
      yyss = (short *) YYSTACK_ALLOC (yystacksize * sizeof (*yyssp));
      __yy_memcpy ((char *)yyss, (char *)yyss1,
		   size * (unsigned int) sizeof (*yyssp));
      yyvs = (YYSTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yyvsp));
      __yy_memcpy ((char *)yyvs, (char *)yyvs1,
		   size * (unsigned int) sizeof (*yyvsp));
#ifdef YYLSP_NEEDED
      yyls = (YYLTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yylsp));
      __yy_memcpy ((char *)yyls, (char *)yyls1,
		   size * (unsigned int) sizeof (*yylsp));
#endif
#endif /* no yyoverflow */

      yyssp = yyss + size - 1;
      yyvsp = yyvs + size - 1;
#ifdef YYLSP_NEEDED
      yylsp = yyls + size - 1;
#endif

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Stack size increased to %d\n", yystacksize);
#endif

      if (yyssp >= yyss + yystacksize - 1)
	YYABORT;
    }

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Entering state %d\n", yystate);
#endif

  goto yybackup;
 yybackup:

/* Do appropriate processing given the current state.  */
/* Read a lookahead token if we need one and don't already have one.  */
/* yyresume: */

  /* First try to decide what to do without reference to lookahead token.  */

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* yychar is either YYEMPTY or YYEOF
     or a valid token in external form.  */

  if (yychar == YYEMPTY)
    {
#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Reading a token: ");
#endif
      yychar = YYLEX;
    }

  /* Convert token to internal form (in yychar1) for indexing tables with */

  if (yychar <= 0)		/* This means end of input. */
    {
      yychar1 = 0;
      yychar = YYEOF;		/* Don't call YYLEX any more */

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Now at end of input.\n");
#endif
    }
  else
    {
      yychar1 = YYTRANSLATE(yychar);

#if YYDEBUG != 0
      if (yydebug)
	{
	  fprintf (stderr, "Next token is %d (%s", yychar, yytname[yychar1]);
	  /* Give the individual parser a way to print the precise meaning
	     of a token, for further debugging info.  */
#ifdef YYPRINT
	  YYPRINT (stderr, yychar, yylval);
#endif
	  fprintf (stderr, ")\n");
	}
#endif
    }

  yyn += yychar1;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != yychar1)
    goto yydefault;

  yyn = yytable[yyn];

  /* yyn is what to do for this token type in this state.
     Negative => reduce, -yyn is rule number.
     Positive => shift, yyn is new state.
       New state is final state => don't bother to shift,
       just return success.
     0, or most negative number => error.  */

  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrlab;

  if (yyn == YYFINAL)
    YYACCEPT;

  /* Shift the lookahead token.  */

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting token %d (%s), ", yychar, yytname[yychar1]);
#endif

  /* Discard the token being shifted unless it is eof.  */
  if (yychar != YYEOF)
    yychar = YYEMPTY;

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  /* count tokens shifted since error; after three, turn off error status.  */
  if (yyerrstatus) yyerrstatus--;

  yystate = yyn;
  goto yynewstate;

/* Do the default action for the current state.  */
yydefault:

  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;

/* Do a reduction.  yyn is the number of a rule to reduce with.  */
yyreduce:
  yylen = yyr2[yyn];
  if (yylen > 0)
    yyval = yyvsp[1-yylen]; /* implement default value of the action */

#if YYDEBUG != 0
  if (yydebug)
    {
      int i;

      fprintf (stderr, "Reducing via rule %d (line %d), ",
	       yyn, yyrline[yyn]);

      /* Print the symbols being reduced, and their result.  */
      for (i = yyprhs[yyn]; yyrhs[i] > 0; i++)
	fprintf (stderr, "%s ", yytname[yyrhs[i]]);
      fprintf (stderr, " -> %s\n", yytname[yyr1[yyn]]);
    }
#endif


  switch (yyn) {

case 2:
{ er_yy_tracelog("parsed something\n"); ;
    break;}
case 3:
{
		  MYABORT;
 		;
    break;}
case 4:
{	
			lexerr_t *erst = ypar;
			strncpy(erst->path.name, yyvsp[0].text, 31);
			mycleanup(ypar);
			;
    break;}
case 5:
{ 
			lexerr_t *erst = ypar;
			er_ret_t err;

			erst->path.active = 1;
			err=er_register_path(&(erst->path), erst->path.name);
			mycleanup(ypar);

			if( NOERR(err)) {
				adderr(erst, "created path ");
				adderr(erst, erst->path.name); 
			
			} 
			else {
				adderr(erst, "COULD NOT create path ");
				adderr(erst, erst->path.name);
				switch( err ) {
				    case ER_DUPENT:
					adderr(erst, " - duplicate entry");
					break;
				    case ER_INSANE:
					adderr(erst, " - sanity check not passed");
					break;
				}
				mycleanup(ypar);
				MYABORT;
			}
		     ;
    break;}
case 6:
{ 
			lexerr_t *erst = ypar;
	
			if( NOERR(er_delete_path(yyvsp[0].text)) ) {
			  adderr(erst, " path removed");
			  mycleanup(ypar);	
			}
			else {
			  adderr(erst, " COULD NOT remove path ");
			  MYABORT;
			}
				
		      ;
    break;}
case 7:
{
		lexerr_t *erst = ypar;

		erst->path.active = 1;
		if( NOERR(er_modify_path(&(erst->path), yyvsp[-1].text))) {
				adderr(erst, "modified path ");
				adderr(erst, yyvsp[-1].text); 
				mycleanup(ypar);
			} 
			else {
				adderr(erst, " COULD NOT modify path ");
				adderr(erst, yyvsp[-1].text);
				MYABORT;
			}
		;
    break;}
case 8:
{ 
			lexerr_t *erst = ypar;
			
			if( NOERR(er_delete_filter(yyvsp[-2].text, (unsigned) yyvsp[0].inum))) {	
				adderr(erst, "modified path ");
				adderr(erst, yyvsp[-2].text); 
				mycleanup(ypar);
			}
			else {
				adderr(erst, " COULD NOT remove filter ");
				adderr(erst, "from path ");
				adderr(erst, yyvsp[-2].text);
				MYABORT;	
			}

		 ;
    break;}
case 9:
{
	lexerr_t *erst = ypar;
			
	if( NOERR(er_attach_filter_chain(yyvsp[-2].text, erst->path.filters ))) {	
			adderr(erst, "modified path ");
			adderr(erst, yyvsp[-2].text); 
			mycleanup(ypar);
		}
	else {
	 		adderr(erst, " COULD NOT attach filters to path ");
			adderr(erst, yyvsp[-2].text);
			MYABORT;	
	}
    ;
    break;}
case 10:
{ yyval.text = yyvsp[0].text; ;
    break;}
case 11:
{ yyval.text = "AUTO"/* if AUTO then generate a new one */ ;
    break;}
case 12:
{
			 	  er_yy_tracelog("set path");
		                  ;
    break;}
case 13:
{ ;
    break;}
case 14:
{ ;
    break;}
case 15:
{ ;
    break;}
case 16:
{ lexerr_t *erst = ypar; 
			  adderr(erst, " - wrong PATH keyword"); 
			  MYABORT; ;
    break;}
case 18:
{ lexerr_t *erst = ypar;
			adderr(erst, " - format spec missing"); 
			MYABORT; ;
    break;}
case 21:
{ 
		lexerr_t *erst = ypar;
		int mod = er_getformatval(yyvsp[0].text); 

		if( mod ) {
			erst->path.format |= mod;
			er_yy_tracelog("mode %s = 0x%x\n", yyvsp[0].text, mod); 
			mycleanup(ypar);
		} else {
			adderr(erst, " incorrect mode specification: ");
			adderr(erst, yyvsp[0].text);
			mycleanup(ypar);
			MYABORT; 
		}
	   ;
    break;}
case 22:
{ 
		lexerr_t *erst = ypar;
		erst->path.type = ER_PATH_SOCK;
		erst->path.descr.sock.fd = yyvsp[0].inum;
		er_yy_tracelog("socket. Now check the associated socket\n"); 
	;
    break;}
case 23:
{
			lexerr_t *erst = ypar;
			erst->path.type = ER_PATH_NAME;
			er_yy_tracelog("namespec is %s\n", yyvsp[-1].text); 
			strcpy(erst->path.descr.name.filename, yyvsp[-1].text);
			mycleanup(ypar); 
			;
    break;}
case 24:
{ lexerr_t *erst = ypar;
				adderr(erst, " - filename missing"); 
				MYABORT; ;
    break;}
case 26:
{
			lexerr_t *erst = ypar;
			erst->path.descr.name.date = 1;
		;
    break;}
case 27:
{
			lexerr_t *erst = ypar;
			erst->path.type = ER_PATH_EXEC;	
		 ;
    break;}
case 29:
{
			lexerr_t *erst = ypar;
			erst->path.descr.exec.usepath = 1;
			;
    break;}
case 30:
{ /* first element */
		  lexerr_t *erst = ypar;
		  er_add_exec_arg( &(erst->path), erst->token);
		  mycleanup(ypar); 
		  ;
    break;}
case 31:
{ /* next elements */
		  lexerr_t *erst = ypar;
		  er_add_exec_arg( &(erst->path), erst->token);
		  mycleanup(ypar);
		  ;
    break;}
case 34:
{ er_yy_tracelog("one more filter done\n"); ;
    break;}
case 35:
{ 
			lexerr_t *erst = ypar;
			/* add to list */
			er_add_filter( &erst->path,  &(erst->curfilt)); 
			er_yy_tracelog("added a filter\n"); 	
			memset( &(erst->curfilt), 0, sizeof(er_filter_t) );
			;
    break;}
case 36:
{;
    break;}
case 37:
{ lexerr_t *erst = ypar;
			adderr(erst, " - bad filter spec"); 
			MYABORT; 
		;
    break;}
case 38:
{;
    break;}
case 39:
{;
    break;}
case 41:
{;
    break;}
case 43:
{
			lexerr_t *erst = ypar;
			erst->curfilt.thr_id = pthread_self();
			;
    break;}
case 45:
{ 
		er_yy_tracelog("added fac to mask\n"); 
		;
    break;}
case 46:
{   
		  lexerr_t *erst = ypar;
		  erst->curfilt.fac_mask = er_getfacallmask();
		;
    break;}
case 47:
{ 
		lexerr_t *erst = ypar;
		er_ret_t ef = er_getfacval(yyvsp[0].text); 

		if( ef ) {
			MA_set( &(erst->curfilt.fac_mask), (unsigned) ef, 1);
			er_yy_tracelog("fac %s = %x\n", yyvsp[0].text, ef);
			mycleanup(ypar);
		} else {
			adderr(erst, " incorrect fac specification: ");
			adderr(erst, yyvsp[0].text);
			mycleanup(ypar);
			MYABORT; 
		}
	   ;
    break;}
case 50:
{
			mysetsev(ypar, 0, "D")
			;
    break;}
case 51:
{ if( mysetsev(ypar, 0, yyvsp[0].text) == 0 ) {
			MYABORT; 
		   }
		 ;
    break;}
case 52:
{
			mysetsev(ypar, 1, "F")
         		;
    break;}
case 53:
{ if( mysetsev(ypar, 1, yyvsp[0].text) == 0 ) {
			MYABORT; 	
		   } 
		 ;
    break;}
case 54:
{
		  char *cp = strdup(yyvsp[0].text);
		  if( mysetsev(ypar, 0, yyvsp[0].text) != 0 ) {
		      mysetsev(ypar, 1, cp);
		      mycleanup(ypar);	
		  }
		  else {
			free(cp);
			MYABORT; 	
		   } 
		;
    break;}
case 56:
{ 
			er_yy_tracelog("added onefac\n");
			;
    break;}
case 57:
{ lexerr_t *erst = ypar;
		 er_mask_t em = er_getaspval(yyvsp[0].text);

		 if(em != 0) {
			erst->curfilt.asp_mask |= em;
			er_yy_tracelog("asp %s = %x\n", yyvsp[0].text, em);
			mycleanup(ypar);
		 } 
		 else {
			adderr(erst, " incorrect asp specification: ");
			adderr(erst, yyvsp[0].text);
			mycleanup(ypar);
			MYABORT; 
		      }
		;
    break;}
case 58:
{
		lexerr_t *erst = ypar;
		erst->curfilt.asp_mask |= yyvsp[0].inum;
		mycleanup(ypar);
		;
    break;}
}
   /* the action file gets copied in in place of this dollarsign */


  yyvsp -= yylen;
  yyssp -= yylen;
#ifdef YYLSP_NEEDED
  yylsp -= yylen;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

  *++yyvsp = yyval;

#ifdef YYLSP_NEEDED
  yylsp++;
  if (yylen == 0)
    {
      yylsp->first_line = yylloc.first_line;
      yylsp->first_column = yylloc.first_column;
      yylsp->last_line = (yylsp-1)->last_line;
      yylsp->last_column = (yylsp-1)->last_column;
      yylsp->text = 0;
    }
  else
    {
      yylsp->last_line = (yylsp+yylen-1)->last_line;
      yylsp->last_column = (yylsp+yylen-1)->last_column;
    }
#endif

  /* Now "shift" the result of the reduction.
     Determine what state that goes to,
     based on the state we popped back to
     and the rule number reduced by.  */

  yyn = yyr1[yyn];

  yystate = yypgoto[yyn - YYNTBASE] + *yyssp;
  if (yystate >= 0 && yystate <= YYLAST && yycheck[yystate] == *yyssp)
    yystate = yytable[yystate];
  else
    yystate = yydefgoto[yyn - YYNTBASE];

  goto yynewstate;

yyerrlab:   /* here on detecting error */

  if (! yyerrstatus)
    /* If not already recovering from an error, report this error.  */
    {
      ++yynerrs;

#ifdef YYERROR_VERBOSE
      yyn = yypact[yystate];

      if (yyn > YYFLAG && yyn < YYLAST)
	{
	  int size = 0;
	  char *msg;
	  int x, count;

	  count = 0;
	  /* Start X at -yyn if nec to avoid negative indexes in yycheck.  */
	  for (x = (yyn < 0 ? -yyn : 0);
	       x < (sizeof(yytname) / sizeof(char *)); x++)
	    if (yycheck[x + yyn] == x)
	      size += strlen(yytname[x]) + 15, count++;
	  msg = (char *) malloc(size + 15);
	  if (msg != 0)
	    {
	      strcpy(msg, "parse error");

	      if (count < 5)
		{
		  count = 0;
		  for (x = (yyn < 0 ? -yyn : 0);
		       x < (sizeof(yytname) / sizeof(char *)); x++)
		    if (yycheck[x + yyn] == x)
		      {
			strcat(msg, count == 0 ? ", expecting `" : " or `");
			strcat(msg, yytname[x]);
			strcat(msg, "'");
			count++;
		      }
		}
	      yyerror(msg);
	      free(msg);
	    }
	  else
	    yyerror ("parse error; also virtual memory exceeded");
	}
      else
#endif /* YYERROR_VERBOSE */
	yyerror("parse error");
    }

  goto yyerrlab1;
yyerrlab1:   /* here on error raised explicitly by an action */

  if (yyerrstatus == 3)
    {
      /* if just tried and failed to reuse lookahead token after an error, discard it.  */

      /* return failure if at end of input */
      if (yychar == YYEOF)
	YYABORT;

#if YYDEBUG != 0
      if (yydebug)
	fprintf(stderr, "Discarding token %d (%s).\n", yychar, yytname[yychar1]);
#endif

      yychar = YYEMPTY;
    }

  /* Else will try to reuse lookahead token
     after shifting the error token.  */

  yyerrstatus = 3;		/* Each real token shifted decrements this */

  goto yyerrhandle;

yyerrdefault:  /* current state does not do anything special for the error token. */

#if 0
  /* This is wrong; only states that explicitly want error tokens
     should shift them.  */
  yyn = yydefact[yystate];  /* If its default is to accept any token, ok.  Otherwise pop it.*/
  if (yyn) goto yydefault;
#endif

yyerrpop:   /* pop the current state because it cannot handle the error token */

  if (yyssp == yyss) YYABORT;
  yyvsp--;
  yystate = *--yyssp;
#ifdef YYLSP_NEEDED
  yylsp--;
#endif

#if YYDEBUG != 0
  if (yydebug)
    {
      short *ssp1 = yyss - 1;
      fprintf (stderr, "Error: state stack now");
      while (ssp1 != yyssp)
	fprintf (stderr, " %d", *++ssp1);
      fprintf (stderr, "\n");
    }
#endif

yyerrhandle:

  yyn = yypact[yystate];
  if (yyn == YYFLAG)
    goto yyerrdefault;

  yyn += YYTERROR;
  if (yyn < 0 || yyn > YYLAST || yycheck[yyn] != YYTERROR)
    goto yyerrdefault;

  yyn = yytable[yyn];
  if (yyn < 0)
    {
      if (yyn == YYFLAG)
	goto yyerrpop;
      yyn = -yyn;
      goto yyreduce;
    }
  else if (yyn == 0)
    goto yyerrpop;

  if (yyn == YYFINAL)
    YYACCEPT;

#if YYDEBUG != 0
  if (yydebug)
    fprintf(stderr, "Shifting error token, ");
#endif

  *++yyvsp = yylval;
#ifdef YYLSP_NEEDED
  *++yylsp = yylloc;
#endif

  yystate = yyn;
  goto yynewstate;

 yyacceptlab:
  /* YYACCEPT comes here.  */
  if (yyfree_stacks)
    {
      free (yyss);
      free (yyvs);
#ifdef YYLSP_NEEDED
      free (yyls);
#endif
    }
  return 0;

 yyabortlab:
  /* YYABORT comes here.  */
  if (yyfree_stacks)
    {
      free (yyss);
      free (yyvs);
#ifdef YYLSP_NEEDED
      free (yyls);
#endif
    }
  return 1;
}


#include "er_yacc_helper.inc"
